<?php

namespace WCAldi\Cache;

use WCAldi\Repositories\ProductRepo;
use WCAldi\Repositories\ProductCategoriesRepo;
use WCAldi\Repositories\ProductStoreRepo;

class ProductsCache {

    private $table;
    private $storeId;

    public function __construct() {

        $this->table = $wpdb->prefix . WC_ALDI_PRODUCTS_CACHE;
        $this->storeId = $_COOKIE['current_store']['id'];

    }

    /**
     * It gets the product's from Woocommerce and insert them into a new table acting as cache
     */
    public function saveCache() {

        $products = $this->getProductsFromWoocommerce();
        $this->insertDatabase( $products );

    }

    /**
     * It loads the product's from the Woocommerce database and transform them for the cache table purposes
     * 
     * @return array - An array with all the products prepared to be inserted into the cache database
     */
    public function getProductsFromWoocommerce() {

        $productRepo = new ProductRepo();
        $products = $productRepo->getProductsInfo();

        return $products;

    }

    /**
     * It inserts data into the cache table
     * 
     * @param $products array - Array of products to insert
     */
    public function insertDatabase( $products ) {

        global $wpdb;

        $query = "
            DELETE FROM {$this->table}
        ";

        $wpdb->query($query);

        $insert_products = '';
        $i = 0;

        foreach( $products as $p ) {

            if( $i != 0 ) {
                $insert_products .= ',';
            } else {
                $insert_products .= ' VALUES ';
            }

            $unit_measure = ( $p['unit_measure'] == 0 ) ? 'null' : '"'. $p['unit_measure'] .'"';
            $promo_start_date = ( $p['promo_start_date'] == '' ) ? 'null' : '"'. $p['promo_start_date'] .'"';
            $promo_end_date = ( $p['promo_end_date'] == '' ) ? 'null' : '"'. $p['promo_end_date'] .'"';
            $promo_normal_price = ( $p['promo_normal_price'] == '' ) ? 'null' : '"'. $p['promo_normal_price'] .'"';
            $promo_offer_price = ( $p['promo_offer_price'] == '' ) ? 'null' : '"'. $p['promo_offer_price'] .'"';

            $seals = [];
            foreach ( $p['seals'] as $s ) {
                $seals[] = $s['img']['url'];
            }

            $insert_products .= '(
                '  . $p['id'] . ',
                "' . $p['name'] . '",
                "' . $p['description'] . '",
                "' . $p['product_modified'] . '",
                "' . $p['variety'] . '",
                "' . $p['origin'] . '",
                "' . $p['brand'] . '",
                ' . $unit_measure . ',
                "' . $p['unit_content'] . '",
                "' . $p['legal_text'] . '",
                "' . $p['nutritional_category'] . '",
                "' . $p['pack'] . '",
                "' . $p['outstanding_sentence'] . '",
                "' . $p['sku'] . '",
                "' . $p['product_family'] . '",
                "' . $p['featured_img'] . '",
                "' . implode( ",", $p['images'] ) . '",
                "' . $p['permalink'] . '",
                "' . $p['order'] . '",
                "' . $p['metatags']['meta_keywords'] . '",
                "' . $p['metatags']['meta_description'] . '",
                "' . $p['product_type'] . '",
                "' . $p['aldi_family'] . '",
                "' . $p['aldi_code'] . '",
                "' . implode( ",", $seals ) . '",
                ' . $promo_start_date . ',
                ' . $promo_end_date . ',
                ' . $promo_normal_price . ',
                ' . $promo_offer_price . ',
                "' . $p['promo']['normal_unit_price'] . '",
                "' . $p['promo']['offer_unit_price'] . '",
                "' . $p['visibility'] . '"
            )';

            $i++;

        }

        $query = "
        INSERT
            INTO {$this->table}
                (product_id,
                name,
                description,
                date,
                variety,
                origin,
                brand,
                unit_measure,
                unit_content,
                legal_text,
                nutritional_category,
                pack,
                outstanding_sentence,
                sku,
                product_family,
                featured_img,
                images,
                permalink,
                order_product,
                meta_keywords,
                meta_description,
                product_type,
                aldi_family,
                aldi_code,
                seals,
                promo_start_date,
                promo_end_date,
                promo_normal_price,
                promo_offer_price,
                promo_normal_unit_price,
                promo_offer_unit_price,
                visibility)

                /* Product's info */
                {$insert_products}
        ";

        return $wpdb->query($query);

    }

    /**
     * It says if the cache is activate or not
     * 
     * @return boolean
     */
    public function isCacheActive() {

        $products_cache_options = get_option( 'products_cache_options' );

        return ( isset( $products_cache_options['products_cache_activate'] ) && $products_cache_options['products_cache_activate'] === 'products_cache_activate' ) ? true : false;
    
    }

    /**
     * It gets the subcategories of a category, and adds the products that belong to them
     *
     * @param int $category_id - Id of the category
     * @param int $limit - Maximum number of elements to retrieve - null for no limit - Default null
     * @param int $offset - Where to start getting the info from - Default 0
     * @param string $orderby - How to sort the products - alphabetical, alphabetical_inverse, price, price_inverse - Default alphabetical
     * @param array $families - Families to retrieve products from - Empty array for all of them - Default empty
     *
     * @return array
     */
    public function getProductsInCategories( $category_id, $limit = null, $offset = 0, $orderby = 'alphabetical', $families = [] ) {
        
        $productsCat = [];

        if( $this->storeId != '' ) {

            $productCategoriesRepo = new ProductCategoriesRepo();

            $categories = $productCategoriesRepo->getSubcategories( $category_id );

            $products = $this->getProductsInCategoriesBBDD( $category_id, $limit, $offset, $orderby, $families );

            if( !empty( $categories ) ) {

                $i = 0;

                foreach( $categories as $category ) {

                    $productsCat[$i]['name'] = $category['name'];
                    $productsCat[$i]['id'] = $category['id'];
                    $productsCat[$i]['slug'] = $category['slug'];
                    
                    if( !empty( $category['subcategories'] ) ) {

                        $j = 0;
                        $isSubcategoryEmpty = true;
        
                        foreach ( $category['subcategories'] as $subcategory ) {

                            $productsSubcategory = [];

                            foreach( $products as $product ) {

                                if( $product['category_id'] == $subcategory['id'] ) {
                                    $productsSubcategory[] = $product;
                                }

                            }

                            if( !empty( $productsSubcategory ) ) {
                                
                                $productsCat[$i]['subcategories'][$j]['name'] = $subcategory['name'];
                                $productsCat[$i]['subcategories'][$j]['id'] = $subcategory['id'];
                                $productsCat[$i]['subcategories'][$j]['slug'] = $subcategory['slug'];
                                $productsCat[$i]['subcategories'][$j]['products'] = $productsSubcategory;

                                $isSubcategoryEmpty = false;

                                $j++;

                            }
        
                        }
        
                    }

                    if( $isSubcategoryEmpty == false ) {
                        $i++;
                    } else {
                        unset( $productsCat[$i] );
                    }
        
                }

            }

        }

        return $productsCat;

    }

    /**
     * It gets all the products belonging to a category and its subcategories
     *
     * @param int $category_id - Id of the category
     * @param int $limit - Maximum number of elements to retrieve - null for no limit - Default null
     * @param int $offset - Where to start getting the info from - Default 0
     * @param string $orderby - How to sort the products - alphabetical, alphabetical_inverse, price, price_inverse - Default alphabetical
     * @param array $families - Families to retrieve products from - Empty array for all of them - Default empty
     *
     * @return array
     */
    public function getProductsInCategoriesBBDD( $category_id, $limit = null, $offset = 0, $orderby = 'alphabetical', $families = [] ) {
        
        global $wpdb;
        
        $products = [];

        if( $this->storeId != '' ) {

            $productStore = new ProductStoreRepo();
            $productRepo = new ProductRepo();

            $query = $this->makeProductsInCategoriesQuery( $category_id, $families );
                
            switch ( $orderby ) {
                case 'alphabetical':
                        $query .= ', posts.name ASC';
                    break;
                case 'alphabetical_inverse':
                    $query .= ', posts.name DESC';
                    break;
                case 'price':
                        $query .= ', ps.price ASC';
                    break;
                case 'price_inverse':
                        $query .= ', ps.price DESC';
                    break;
            }

            if( $limit != null ) {
                $query .= " LIMIT ".$offset.",".$limit;
            }

            if ( $result = $wpdb->get_results( $query, ARRAY_A ) ) {

                foreach( $result as $r ) {

                    $product_id = $r['id'];

                    if( has_post_thumbnail( $product_id ) ) {
                        $thumbnail['url'] = get_the_post_thumbnail_url( $product_id, 'medium');
                        $thumbnail_id = get_post_thumbnail_id( $product_id );
                        $thumbnail['alt'] = get_post_meta($thumbnail_id, '_wp_attachment_image_alt', true);
                    } else {
                        $thumbnail = null;
                    }

                    $products[] = [
                        'id' => $product_id,
                        'name' => $r['name'],
                        'description' => $r['description'],
                        'permalink' => get_the_permalink( $product_id ),
                        'price' => $r['price'],
                        'price_unit' => $r['price_unit'],
                        'content_unit' => strtolower( $productRepo->getAttribute($product_id, 'Pack') ),
                        'thumbnail' => $thumbnail,
                        'category_id' => $r['category_id'],
                        'category_name' => $r['category_name']
                    ];

                    $i++;

                }

            }

        }

        return $products;

    }

    /**
     * It counts how many products would be for the query without pagination
     *
     * @param int $category_id
     * @param string $families - String with the id of the families
     */
    public function countProductsInCategories( $category_id, $families = [] ) {
        
        global $wpdb;

        if( $this->storeId != '' ) {

            $query = $this->makeProductsInCategoriesQuery( $category_id, $families, true );

            if ( $result = $wpdb->get_results( $query, ARRAY_A ) ) {
                return $result[0]['total'];
            }

        } else {
            return 0;
        }

        return 0;

    }

    /**
     * Building a query for getting the products in the categories query
     *
     * @param int $category_id
     * @param array $families - Array with the ids of the families
     * @param boolean $onlyCount - Only count instead of getting the values - Default false
     *
     * @return string - The query
     */
    public function makeProductsInCategoriesQuery( $category_id, $families, $onlyCount = false ) {
        
        global $wpdb;

        if( $onlyCount ) {
            $data = "COUNT(posts.product_id) as `total`";
        } else {
            $data = "posts.product_id AS `id`,
                posts.name AS `name`,
                posts.description AS `description`,
                ps.price AS `price`,
                ps.price_unit AS `price_unit`,
                term_taxonomy.term_id AS `category_id`,
                {$wpdb->prefix}terms.name AS `category_name`";
        }

        if( empty( $families ) ) {
            $query_families = "AND parent IN ( SELECT term_id FROM {$wpdb->prefix}term_taxonomy WHERE parent = {$category_id} )";
        } else {
            $query_families = "AND {$wpdb->prefix}terms.term_id IN ( " . implode(",", $families) . " )";
        }

        $query = "
            SELECT
                {$data}
            FROM
                {$wpdb->prefix}term_taxonomy AS term_taxonomy
            LEFT JOIN
                {$wpdb->prefix}terms ON term_taxonomy.term_id = {$wpdb->prefix}terms.term_id
            LEFT JOIN
                {$wpdb->prefix}term_relationships AS term_relationships ON term_relationships.term_taxonomy_id = term_taxonomy.term_id
            LEFT JOIN
                {$this->table} AS posts ON term_relationships.object_id = posts.product_id
            LEFT JOIN
                {$wpdb->prefix}wcaldi_product_store AS ps ON ps.product_id = posts.product_id
            LEFT JOIN
                {$wpdb->prefix}termmeta as tm ON tm.term_id = {$wpdb->prefix}terms.term_id
                    AND tm.meta_key = 'order'
            WHERE
                taxonomy = 'product_cat'
                {$query_families}
                AND ( ( SELECT is_available
                        FROM {$wpdb->prefix}wcaldi_product_store AS ps
                        WHERE ps.product_id = posts.product_id
                        AND ps.store_id = {$this->storeId} ) = 1 )
                AND ps.store_id = {$this->storeId}
            ORDER BY
                LENGTH(tm.meta_value), tm.meta_value";

        return $query;

    }

    /**
     * It gets the info of a product
     * 
     * @param $product_id
     * 
     * @return array
     */
    public function getProductInfo( $product_id ) {

        global $wpdb;

        $sql = "SELECT * FROM {$this->table} WHERE product_id = {$product_id}";

        return $wpdb->get_results( $sql, ARRAY_A )[0];

    }

}