<?php

namespace WCAldi\Exporter;

use WCAldi\Repositories\ProductRepo;

class ProductsExporter {

    public function init() {

        // Execute CRON
        add_action('products_json_cron', [ $this, 'generateJson' ] );

        //add_action('init', [ $this, 'generateJson' ] );

    }

    /**
     * It gets the array with product's info from Woocommerce and parse it for the App
     * 
     * @param array $products
     * 
     * @return array
     */
	public function parseArrayForApp( $products ) {

        $arrayProducts = [];

        foreach( $products as $p ) {

            $promos = [
                'start_date' => '',
                'end_date' => '',
                'view_start_date' => '',
                'view_end_date' => '',
                'normal_price' => '',
                'offer_price' => '',
                'normal_unit_price' => '',
                'offer_unit_price' => ''
            ];

            if( isset( $p['promo'] ) ) {
                
                $promos = [
                    'start_date' => $p['promo']['start_date'],
                    'end_date' => $p['promo']['end_date'],
                    'view_start_date' => $p['promo']['view_start_date'],
                    'view_end_date' => $p['promo']['view_end_date'],
                    'normal_price' => $p['promo']['normal_price'],
                    'offer_price' => $p['promo']['offer_price'],
                    'normal_unit_price' => $p['promo']['normal_unit_price'],
                    'offer_unit_price' => $p['promo']['offer_unit_price']
                ];

            }

            // Setting the product type
            $product_type = "fijo";
            if( $p['product_type'] === 'Exprés' ) {
                $product_type = "express";
            } else if ( $p['product_type'] === 'Inandout') {
                $product_type = "inandout";
            } else if ( $p['product_type'] === 'Superlike') {
                $product_type = "superlike";
            }

            if( $p['aldi_family'] == 'AA' ) {
                $product_number = "2".str_pad( $p['aldi_code'], 4, '0', STR_PAD_LEFT );
            } else {
                $product_number = "1".str_pad( $p['aldi_code'], 4, '0', STR_PAD_LEFT );
            }

            $seals = [];
            foreach( $p['seals'] as $s ) {
                $seals[] = $s['img']['url'];
            }

            $arrayProducts[] = [
                "priceZona3" => "",
                "freshPack" => "",
                "priceZona4" => "",
                "metaTagKeywords" => $p['metatags']['meta_keywords'],
                "priceZona1" => "",
                "priceZona2" => "",
                "viewStartDate" => ( $promos['view_start_date'] != '' ) ? intval( ( strtotime( $promos['view_start_date'] ) - 3600 ) . '000' ) : '',
                "productDescription" => $p['description'],
                "priceZona5" => "",
                "endDate" => ( $promos['end_date'] != '' ) ? intval( ( strtotime( $promos['end_date'] ) - 3600 ) . '000' ) : '',
                "pack" => $p['pack'],
                "priceUnit" => $p['price'],
                "related" => "true",
                "productType" => $product_type,
                "photoFileName" => $p['featured_img'],
                "priceUnitZona2" => "",
                "outstandingSentenceExpress" => "",
                "priceUnitZona1" => "",
                "priceUnitZona4" => "",
                "family" => $p['aldi_family'],
                "priceUnitZona3" => "",
                "testimonial" => "",
                "linkToProduct" => $p['permalink'],
                "imageGallery" => $p['images'],
                "expiredDate" => "",
                "priceUnitZona5" => "",
                "category" => $p['categories'],
                "price" => $p['price'],
                "unitMeasure" => $p['unit_measure'],
                "region" => "",
                "startDate" => ( $promos['start_date'] != '' ) ? intval( ( strtotime( $promos['start_date'] ) - 3600 ) . '000' ) : '',
                "outstandingSentence" => $p['outstanding_sentence'],
                "topTen" => "false",
                "extraNumeric2" => "",
                "extraNumeric1" => "",
                "id" => $p['aldi_family']."-".$product_number,
                "productNumber" => $product_number,
                "viewEndDate" => ( $promos['view_end_date'] != '' ) ? intval( ( strtotime( $promos['view_end_date'] ) - 3600 ) . '000' ) : '',
                "seal" => implode( ',', $seals ),
                "fresh" => ( $p['aldi_family'] == 'WG44' || $p['aldi_family'] == 'WG46' ) ? 'true' : 'false' ,
                "Image" => "",
                "sortValue" => $p['order'],
                "extraText2" => "",
                "extraText1" => "",
                "origin" => $p['origin'],
                "activoZona4" => "false",
                "activoZona5" => "false",
                "activoZona2" => "false",
                "activoZonaDefault" => "true",
                "activoZona3" => "false",
                "friendlyPath" => "",
                "activoZona1" => "false",
                "netWeight" => "",
                "unit" => $p['unit_content'],
                "Text" => "",
                "Link" => "",
                "metaTagDescription" => $p['metatags']['meta_description'],
                "legalText" => $p['legal_text'],
                "brand" => $p['brand'],
                "productFamily" => $p['web_family'],
                "nutritionalInformation" => $p['nutritional_category'],
                "productName" => $p['name'],
                "aldiWorld" => ""
            ];

        }
        
        return $arrayProducts;

    }

    /**
     * It generates a JSON with all the info required by the APP
     */
    public function generateJson() {
        
        $fileName = WC_ALDI_PLUGIN_PATH . 'json/products.json';
        
        $productRepo = new ProductRepo();
        $products    = $productRepo->getProductsInfo( false );
    
        $arrayForApp      = $this->parseArrayForApp( $products );
        $jsonString       = json_encode( $arrayForApp );
    
        file_put_contents( $fileName, $jsonString );

    }

    /**
     * Get products JSON file content
     */
    public function getJsonFromFile() {

        $fileName = WC_ALDI_PLUGIN_PATH . 'json/products.json';

        if( file_exists( $fileName ) ) {
            return file_get_contents($fileName);
        } else {
            return json_encode( [
                'error' => 'No se ha podido encontrar el fichero JSON.'
            ] );
        }

    }
    
}