<?php

namespace WCAldi\Managers;

class ProductPromoManager {

	/**
	 * Init manager
	 */
	public function init() {
		$this->applyHooks();
	}

	/**
	 * It calls the hooks for the tab
	 */
	protected function applyHooks() {
		// Creating a tab within the product backend
		add_filter( 'woocommerce_product_data_tabs', [ $this, 'addPromosTabToProductBackend' ], 10, 2 );
        add_action( 'admin_enqueue_scripts', [ $this, 'stylePromoTabProductBackend' ] );
		add_action( 'woocommerce_product_data_panels', [ $this, 'promosTabContent'] );
		add_action( 'save_post', [ $this, 'savingPromosTabProduct' ], 10, 2 );
        
	}

	/**
     * Creates the section for stores beneath the products tab
     */
    public function addPromosTabToProductBackend( $tabs ) {
        
        $tabs['wcaldi_product_promos'] = array(
            'label'		=> __( 'Promociones', 'wcaldi' ),
            'target'	=> 'product_promos_data',
            'class'     => array( 'product_promos_tab' )
        );
        return $tabs;
        
    }

    /**
     * Adding style to the tab
     */
    public function stylePromoTabProductBackend() {

		// Scripts
		wp_enqueue_script( 'promos-tab-product-backend', WC_ALDI_PLUGIN_URL . 'assets/js/product_backend.js', [], '4.9.3' );
		// SCRIPTS
		wp_localize_script( 'promos-tab-product-backend', 'WC_ALDI', array(

			'name_tax' => WC_ALDI_TAX_PROMOS,

		));
    }

	/**
	 * HTML content for the Stores tab
	 */
	public function promosTabContent() {

		global $post;
		
        $params['promosProductArray'] = [
			'startdate'         => get_post_meta( $post->ID, 'wcaldi_promo_startdate', true),
			'enddate'           => get_post_meta( $post->ID, 'wcaldi_promo_enddate', true),
			'normal_price'      => get_post_meta( $post->ID, 'wcaldi_promo_normal_price', true),
			'offer_price'       => get_post_meta( $post->ID, 'wcaldi_promo_offer_price', true),
			'normal_unit_price' => get_post_meta( $post->ID, 'wcaldi_promo_normal_unit_price', true),
			'offer_unit_price'  => get_post_meta( $post->ID, 'wcaldi_promo_offer_unit_price', true),
			'promo_discount'    => get_post_meta( $post->ID, 'wcaldi_promo_discount', true),
			'promo_palet'       => get_post_meta( $post->ID, 'wcaldi_promo_palet', true)
		];

		echo wcaldi_get_module( 'backend/productTabs/promosTabProductBackend', $params);

	}

	/**
	 * It saves the data of the Stores tab
	 */
	public function savingPromosTabProduct( $post_id ) {

		// Getting out of the function if it's not a product
		if ( get_post_type( $post_id ) != "product" ) {
			return;
		}
		
		if( isset( $_POST['wcaldi_promo']) ) {
			// Updating meta data
			update_post_meta( $post_id, 'wcaldi_promo_startdate', $_POST['wcaldi_promo']['startdate']);
			update_post_meta( $post_id, 'wcaldi_promo_enddate', $_POST['wcaldi_promo']['enddate']);
			update_post_meta( $post_id, 'wcaldi_promo_normal_price', $_POST['wcaldi_promo']['normal_price']);
			update_post_meta( $post_id, 'wcaldi_promo_offer_price', $_POST['wcaldi_promo']['offer_price']);
			update_post_meta( $post_id, 'wcaldi_promo_normal_unit_price', $_POST['wcaldi_promo']['normal_unit_price']);
			update_post_meta( $post_id, 'wcaldi_promo_offer_unit_price', $_POST['wcaldi_promo']['offer_unit_price']);
			update_post_meta( $post_id, 'wcaldi_promo_discount', $_POST['wcaldi_promo']['promo_discount']);
			update_post_meta( $post_id, 'wcaldi_promo_palet', $_POST['wcaldi_promo']['promo_palet']);
		}

	}
}