<?php

namespace WCAldi\Managers;
use WCAldi\Repositories\ProductStoreRepo;
use WCAldi\Repositories\StoreRepo;

class ProductStoreManager {

	protected $storeId;

	public function __construct() {

		if( isset( $_COOKIE['current_store']['id'] ) ) {
			$this->storeId = $_COOKIE['current_store']['id'];
		}

	}

	/**
	 * Init manager
	 */
	public function init() {
		$this->applyFilterByAvailability();
		$this->applyProductPrice();
		$this->applyTabFilters();
	}

	/**
	 * It checks the availabilyty of a product and decides whether showing it
	 */
	protected function applyFilterByAvailability() {
		add_filter( 'posts_join', [ $this, 'applyFilterByAvailabilityJoin' ], 10, 2 );
		add_filter( 'posts_where', [ $this, 'applyFilterByAvailabilityWhere' ], 10, 2 );      
	}

	/**
	 * It calls the tab filters
	 */
	protected function applyTabFilters() {
		// Creating a tab within the product backend
		add_filter( 'woocommerce_product_data_tabs', [ $this, 'addStoreTabToProductBackend' ], 10, 2 );
		add_action( 'woocommerce_product_data_panels', [ $this, 'storeTabContent'] );
		add_action( 'save_post', [ $this, 'savingStoreTabProduct' ] );
	}

	/**
	 * Only shows product when has store information
	 *
	 * @param $join
	 * @param $query
	 *
	 * @return string
	 */
	public function applyFilterByAvailabilityJoin( $join, $query ) {
		if ( $this->areFiltersApplicable( $query ) ) {
			global $wpdb;
			$join .= " JOIN " . WC_ALDI_PRODUCT_STORE_TABLE . " wcaldi ON " .
			         $wpdb->posts . ".id = wcaldi.product_id AND wcaldi.store_id={$this->storeId} ";
		}

		return $join;
	}

	/**
	 * Only shows product when is_available field is true in table wcaldi_product_store
	 *
	 * @param $join
	 * @param $query
	 *
	 * @return string
	 */
	public function applyFilterByAvailabilityWhere( $where, $query ) {
		if ( $this->areFiltersApplicable( $query ) ) {
			$where .= " AND wcaldi.is_available=1 ";
		}

		return $where;
	}

	/**
	 * Determines when is a WooCommerce query and have to apply product store filters
	 *
	 * @param $query
	 *
	 * @return bool
	 */
	protected function areFiltersApplicable( $query ) {
		return ! is_admin()
		       && $query->get( 'post_type' ) == 'product'
		       && ! empty( $query->get( 'wc_query' ) );
	}

	/**
	 * It shows the price of a product depending on the store price
	 */
	public function applyProductPrice() {

		add_filter( 'woocommerce_get_price_html', function ( $price ) {
			global $wpdb, $post;

			if( $this->storeId ) {
				$query   = "SELECT * FROM " . WC_ALDI_PRODUCT_STORE_TABLE . " WHERE product_id = {$post->ID} AND store_id = {$this->storeId}";
				$results = $wpdb->get_results( $query, OBJECT );

				if ( ! empty( $results ) ) {
					$price = $results[0]->price;
				} else {
					$price = null;
				}

				return $price;

			}

		} );

	}

	/**
     * Creates the section for stores beneath the products tab
     */
    public function addStoreTabToProductBackend( $tabs ) {
        
        $tabs['wcaldi_product_store'] = array(
            'label'		=> __( 'Datos por tienda', 'wcaldi' ),
            'target'	=> 'product_store_data',
            'class'     => array( 'product_store_tab' )
        );
        return $tabs;
        
    }

	/**
	 * HTML content for the Stores tab
	 */
	public function storeTabContent() {

		global $post;

		$storeProducts = new ProductStoreRepo();
		$params['storeProductsArray'] = $storeProducts->getByProductId( $post->ID );

		echo wcaldi_get_module( 'backend/productTabs/storeTabProductBackend', $params);

	}


	/**
	 * It saves the data of the Stores tab
	 */
	public function savingStoreTabProduct( $postId ) {

		if ( get_post_type( $postId ) != 'product' ) {
			return;
		}

		if ( !isset( $_POST['wcaldi_stores'] ) ) {
			return;
		}

		// Fill data to be inserted.
		$data = [];

		foreach ( $_POST['wcaldi_stores'] as $item ) {

			if ($item['price']) {

				$isAvailable = $item['availability'] == 'on';
				$price       = str_replace(',', '.', $item['price']);
				$priceUnit   = str_replace(',', '.', $item['price_unit']);

				if (!$priceUnit) {
					$priceUnit = null;
				}


				$data[] = array(
					'product_id'   => $postId,
					'store_id'     => $item['store_id'],
					'is_available' => $isAvailable,
					'price'        => $price,
					'price_unit'   => $priceUnit
				);

			}
			
		}

		// Remove old prices and add the new ones.
		$productStoreRepo = new ProductStoreRepo();
		$productStoreRepo->deleteByProductId( $postId );
		$productStoreRepo->insertData( $data );

	}

}

