<?php

namespace WCAldi\Managers;
use WCAldi\Repositories\ProductStoreRepo;
use WCAldi\Repositories\StoreRepo;
use WCAldi\Repositories\PromosRepo;

class StoreSelectorManager {

	private $optionName = 'available_stores';

    public function __construct() {

    }

    /**
	 * Init manager
	 */
	public function init() {
        $this->storeCookieManager();
		$this->applyStoreSelectorHooks();
	}

    /**
	 * Store selector hooks
	 */
	protected function applyStoreSelectorHooks() {
		
		add_action( 'wp_footer', [ $this, 'addStoreSelectorModal' ] );
		add_action( 'init', [ $this, 'loadAjaxHandlers'] );
		add_action( 'wp_enqueue_scripts', [ $this, 'scriptsStoreSelector' ] );
        add_action( 'widgets_init', [ $this, 'wpb_load_widget' ] );

	}

	/**
	 * Loading AJAX Handlers
	 */
	public function loadAjaxHandlers() {

		// AJAX Handlers
		add_action( 'wp_ajax_loadClosestsStores', array( $this, 'loadClosestsStores' ));
		add_action( 'wp_ajax_nopriv_loadClosestsStores', array( $this, 'loadClosestsStores' ));

	}

    /**
     * Adding scripts for the store selector
     */
    public function scriptsStoreSelector() {

		wp_enqueue_script( 'maps-google', "https://maps.google.com/maps/api/js?v=3.22&sensor=false&libraries=geometry,places&language=es&key=AIzaSyCGk-wvkrRflib37SXKGMJ3fo4j0UHNAZI", array(), null, true );
		wp_enqueue_script( 'jquery-geocomplete', WC_ALDI_PLUGIN_URL . "vendor/jquery.geocomplete.min.js", array('maps-google'), null, true );
		wp_enqueue_script( 'wcaldi-stores-selector', WC_ALDI_PLUGIN_URL . 'assets/js/store-selector-select.js', array('jquery'), filemtime( WC_ALDI_PLUGIN_PATH . 'assets/js/store-selector.js' ), true );		

		wp_localize_script( 'wcaldi-stores-selector', 'WC_ALDI_StoreLocator', array(

			'pluginurl' => WC_ALDI_PLUGIN_URL,

			// URL to wp-admin/admin-ajax.php to process the request
			'ajaxurl' => admin_url( 'admin-ajax.php' ),

			// generate a nonce with a unique ID "myajax-post-comment-nonce"
			// so that you can check it later when an AJAX request is sent
			'security' => wp_create_nonce( 'store-selector-ajax' )

		));

    }

	/**
	 * Adds a modal at the end of the code with a store selector
	 */
	public function addStoreSelectorModal() {

		if( isset( $_POST['submitFormChangeTienda'] ) ) {
			$params['current_store_id'] = $_POST['idTienda'];
		} else {
			if( isset($_COOKIE['current_store'] ) ) {
				$params['current_store_id'] = $_COOKIE['current_store']['id'];
			} else {
				$params = [];
			}
		}

		$params['stores'] = $this->getStores();

		echo wcaldi_get_module( 'frontend/storeSelector/storeSelectorModal-Select', $params);

	}

	/**
	 * Load stores
	 */
	public function loadClosestsStores() {

		check_ajax_referer( 'store-selector-ajax', 'security' );
		
		$stores = new StoreRepo();
		$storesArray = $stores->getAllInfo();

		$selectedLatitude = $_POST['latitude'];
		$selectedLongitude = $_POST['longitude'];

		$ref = [ $selectedLatitude, $selectedLongitude ];

		$storesForSelector = [];

		$distances = array_map( function($item) use($ref) {
			$a = [ $item['latitude'], $item['longitude'] ];
			return $this->distance($a, $ref);
		}, $storesArray);

		asort($distances);
		
		for( $i=0; $i<5; $i++ ) {

			$s = $storesArray[key($distances)];

			$storesForSelector[] = [
				'IdTienda' 				=> $s['id'],
				'NombreTienda' 			=> $s['name'],
				'CodigoTienda' 			=> $s['code'],
				'DistanciaTienda' 		=> current($distances),
				'CodigoPostalTienda' 	=> $s['postal_code'],
				'ProvinciaTienda' 		=> $s['province'],
				'CodigoPlataforma' 		=> $s['platform'],
				'isBaleares'			=> $s['isBaleares'],
				'latitude'				=> $s['latitude'],
				'longitude'				=> $s['longitude']
			];

			next( $distances );

		}

		echo json_encode( $storesForSelector );

		die();

	}

    /**
     * It selects and changes the current store and saves it to a cookie
     */
    public function storeCookieManager() {

        // If we get info of a store, we save it in a cookie
        if( isset( $_POST['submitFormChangeTienda'] ) ) {

			$expiration = time()+31556926;

			$home_url = home_url();
			
			setcookie( 'current_store_id', $_POST['idTienda'], $expiration, '/' ); // This cookie is used only for Pagely cache's purposes
			setcookie( 'current_store[id]', $_POST['idTienda'], $expiration, '/' );
			setcookie( 'current_store[code]', $_POST['codTienda'], $expiration, '/' );
			setcookie( 'current_store[name]', $_POST['nombreTienda'], $expiration, '/' );
			setcookie( 'current_store[location]', $_POST['ubicacion'], $expiration, '/' );
			setcookie( 'current_store[isBaleares]', $_POST['isBaleares'], $expiration, '/' );
			setcookie( 'current_store[longitude]', $_POST['longitude'], $expiration, '/' );
			setcookie( 'current_store[latitude]', $_POST['latitude'], $expiration, '/' );
			setcookie( 'current_store[postal_code]', $_POST['postal_code'], $expiration, '/' );

			header('Location: ' . home_url( add_query_arg( NULL, NULL ) ) ) ;
			exit;

        }

    }

	/**
	 * It returns the distance between two points in kilometers
	 */
	private function distance($a, $b) {
		list($lat1, $lon1) = $a;
		list($lat2, $lon2) = $b;

		$theta = $lon1 - $lon2;
		$dist = sin(deg2rad($lat1)) * sin(deg2rad($lat2)) +  cos(deg2rad($lat1)) * cos(deg2rad($lat2)) * cos(deg2rad($theta));
		$dist = acos($dist);
		$dist = rad2deg($dist);
		$km = $dist * 60 * 1.1515 * 1.609344;

		return $km;
	}

    /**
     * WIDGET
     *
     * Functions for creating a widget for the button
     */
    // Register and load the widget
    public function wpb_load_widget() {
        register_widget( 'WCAldi\Widgets\StoreSelectorWidget' );
	}
	
	/**
	 * It saves all the stores in an options field of Wordpress
	 */
	public function saveStores() {

		$storeRepo = new StoreRepo();
		$stores = $storeRepo->getAllInfo(null,'title');

		update_option( $this->optionName, serialize( $stores ) );

	}

	/**
	 * It recovers the available stores
	 * 
	 * @return array
	 */
	public function getStores() {

		return unserialize( get_option( $this->optionName ) );

	}

}