<?php
/**
 * Generated by the WordPress Option Page generator
 * at http://jeremyhixon.com/wp-tools/option-page/
 */

namespace WCAldi\OptionPages;

use WCAldi\MassiveImporter\MassiveProductsImporter;
use WCAldi\Exporter\ProductsExporter;

class ProductsJsonGeneratorOptionPage {

	private $products_json_options, $menu_hook;

	public function init() {

		add_action( 'admin_menu', array( $this, 'products_json_add_plugin_page' ) );
		add_action( 'admin_init', array( $this, 'products_json_page_init' ) );

		// Datepicker
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_styles_scripts' ) );

		// "Generate now" Button
		add_action( 'wp_ajax_generateJsonNow', array( $this, 'generateJsonNow' ));
		add_action( 'wp_ajax_nopriv_generateJsonNow', array( $this, 'generateJsonNow' ));

	}

	public function products_json_add_plugin_page() {

		$this->menu_hook = add_submenu_page(
			'edit.php?post_type=product', // parent_slug
			'JSON Productos', // page_title
			'JSON Productos', // menu_title
			'manage_options', // capability
			'json-productos', // menu_slug,
			array( $this, 'products_json_create_admin_page' ) // function
		);

		// Creating cron on save
		add_action( 'load-'.$this->menu_hook, array( $this,'products_json_schedule_on_save' ) );
	}

	public function products_json_create_admin_page() {
		$this->products_json_options = get_option( 'products_json_options' ); ?>

		<div class="wrap">
            <h2>JSON de productos</h2>
            <?php echo date('H:i:s'); ?>
            <p>La APP de Aldi se alimenta parte de IMA y parte de la página web en la que te encuentras. Este JSON dispone de un fichero que sirve como caché para agilizar el proceso de importación de datos de la APP.</p>
            <p>Desde aquí puedes configurar cuándo se generará el JSON de manera automática.</p>
			<?php settings_errors(); ?>

			<form method="post" action="options.php">
				<?php
					settings_fields( 'products_json_option_group' );
					do_settings_sections( 'json-productos-admin' );
					// Tiempo de cron
					if( wp_next_scheduled('products_json_cron') ) { // Getting next time the cron will run
					  echo "El JSON volverá a generarse automáticamente el <strong>".date("d/m/Y - G:i",wp_next_scheduled('products_json_cron'))."</strong><br>Deja vacía la fecha para evitar que se genere automáticamente.";
					} else {
					  echo "<div class='alert'>No hay una fecha esperada para generar el JSON.<br><strong>Por favor, rellene los campos superiores y presione 'Guardar cambios'.</strong></div>";
					}

					submit_button();
				?>
			</form>

			<div class="products_json_options_page">
				<div class="products_json_options_page_loading">Generando JSON... Este proceso continuará ejecutándose en segundo plano y podría tardar. <img src="<?php echo wcaldi_get_asset("img/backend/loader.gif"); ?>"></div>
				<button class="products_json_options_page_import">Generar ahora</button>
				<div class="products_json_options_page_messages"></div>
			</div>

		</div>
	<?php }

	public function products_json_page_init() {

		register_setting(
			'products_json_option_group', // option_group
			'products_json_options', // option_name
			array( $this, 'products_json_sanitize' ) // sanitize_callback
		);

		add_settings_section(
			'products_json_setting_section', // id
			'Settings', // title
			array( $this, 'products_json_section_info' ), // callback
			'json-productos-admin' // page
		);

		add_settings_field(
			'products_json_periodicity', // id
			'Periodicidad', // title
			array( $this, 'periodicity_callback' ), // callback
			'json-productos-admin', // page
			'products_json_setting_section' // section
		);

		add_settings_field(
			'products_json_start_date', // id
			'Fecha de comienzo', // title
			array( $this, 'start_date_callback' ), // callback
			'json-productos-admin', // page
			'products_json_setting_section' // section
		);
		
	}

	public function products_json_sanitize($input) {
		$sanitary_values = array();
		if ( isset( $input['products_json_periodicity'] ) ) {
			$sanitary_values['products_json_periodicity'] = $input['products_json_periodicity'];
		}

		if ( isset( $input['products_json_start_date'] ) ) {
			$sanitary_values['products_json_start_date'] = sanitize_text_field( $input['products_json_start_date'] );
		}

		return $sanitary_values;
	}

	public function products_json_section_info() {
		
	}

	public function products_json_schedule_on_save() {

		if( isset($_GET['settings-updated']) && $_GET['settings-updated'] ) {
			wp_clear_scheduled_hook( 'products_json_cron' );
			$this->products_json_scheduler();
		}

	}

	public function products_json_scheduler() {
		
		$options = get_option( 'products_json_options' );

		if( $options['products_json_periodicity'] ) {
			$periodicity = $options['products_json_periodicity']; // Getting the time range
		} else {
			$periodicity = "daily";
		}

		if( $options['products_json_start_date'] ) {
			$start_date = strtotime( $options['products_json_start_date'] );
		} else {
			$start_date = null;
		}

		if ( ! wp_next_scheduled( 'products_json_cron' ) && $start_date != null ) {
			wp_schedule_event( $start_date, $periodicity, 'products_json_cron' );
		}

	}

	public function periodicity_callback() {
		?> <select name="products_json_options[products_json_periodicity]" id="products_json_periodicity">
			<?php $selected = (isset( $this->products_json_options['products_json_periodicity'] ) && $this->products_json_options['products_json_periodicity'] === 'hourly') ? 'selected' : '' ; ?>
			<option value="hourly" <?php echo $selected; ?>>Cada hora</option>
			<?php $selected = (isset( $this->products_json_options['products_json_periodicity'] ) && $this->products_json_options['products_json_periodicity'] === 'twicedaily') ? 'selected' : '' ; ?>
			<option value="twicedaily" <?php echo $selected; ?>>Dos veces al día</option>
			<?php $selected = (isset( $this->products_json_options['products_json_periodicity'] ) && $this->products_json_options['products_json_periodicity'] === 'daily') ? 'selected' : '' ; ?>
			<option value="daily" <?php echo $selected; ?>> Diariamente</option>
			<?php $selected = (isset( $this->products_json_options['products_json_periodicity'] ) && $this->products_json_options['products_json_periodicity'] === 'weekly') ? 'selected' : '' ; ?>
			<option value="weekly" <?php echo $selected; ?>> Semanalmente</option>
			<?php $selected = (isset( $this->products_json_options['products_json_periodicity'] ) && $this->products_json_options['products_json_periodicity'] === 'monthly') ? 'selected' : '' ; ?>
			<option value="monthly" <?php echo $selected; ?>> Mensualmente</option>
		</select> <?php
	}

	public function start_date_callback() {
		printf(
			'<input class="regular-text datetimepicker" type="text" name="products_json_options[products_json_start_date]" id="products_json_start_date" value="%s">',
			isset( $this->products_json_options['products_json_start_date'] ) ? esc_attr( $this->products_json_options['products_json_start_date']) : ''
		);
    }
	
	// Execute the script to import the products
	public function generateJsonNow() {
	
		check_ajax_referer( 'wc_aldi_json-generator-ajax', 'security' );
        
        $productsExporter = new ProductsExporter();
        $productsExporter->generateJson();

		$return = [
			'status' => 'success',
			'message' => '¡Bien! La generación ha finalizado correctamente.'
		];
		wp_send_json( $return );
		die(); // Wordpress requeriment for using AJAX

	}
	
	/**
	 * Enqueues the built-in Datepicker script
	 * Only loads on the plugin-name settings page
	 */
	function enqueue_styles_scripts( ) {
	
		wp_enqueue_style( 'jquery-admin-datepicker', WC_ALDI_PLUGIN_URL . '/vendor/datepicker/jquery.datetimepicker.min.css', array(), null );
		wp_enqueue_style( 'jquery-admin-json-generator-styles', WC_ALDI_PLUGIN_URL . '/app/OptionPages/ProductsJsonGeneratorOptionPage.css', array(), null );
		wp_enqueue_script( 'jquery-admin-datepicker', WC_ALDI_PLUGIN_URL . '/vendor/datepicker/jquery.datetimepicker.full.min.js', array( 'jquery' ), '', true );
		wp_enqueue_script( 'wc-aldi-json-generator-scripts', WC_ALDI_PLUGIN_URL . '/app/OptionPages/ProductsJsonGeneratorOptionPage.js', array( 'jquery', 'jquery-admin-datepicker' ), '', true );

		wp_localize_script( 'wc-aldi-json-generator-scripts', 'WC_Aldi_json', array(
			// URL to wp-admin/admin-ajax.php to process the request
		
			'ajaxurl' => admin_url( 'admin-ajax.php' ),
		
			// generate a nonce with a unique ID "myajax-post-comment-nonce"
			// so that you can check it later when an AJAX request is sent
			'security' => wp_create_nonce( 'wc_aldi_json-generator-ajax' )
	
		));
	}

}
	

/* 
 * Retrieve this value with:
 * $products_json_options = get_option( 'products_json_options' ); // Array of All Options
 * $periodicity = $products_json_options['products_json_periodicity']; // Periodicity
 * $start_date = $products_json_options['products_json_start_date']; // Start date
 */
