<?php

namespace WCAldi\PrettyUrl;

use WCAldi\Repositories\ProductCategoriesRepo;

class PrettyFilters {

    private $product_category_base;
    private $slug_filters;
    private $slug_families;
    private $slug_order;
    private $slug_page;
    protected $taxonomy;

    public function __construct() {
        
        $this->product_category_base = get_option('woocommerce_permalinks')['category_base'];
        $this->slug_filters = 'filtros-productos';
        $this->slug_families = 'familias-productos';
        $this->slug_order = 'orden-productos';
        $this->slug_page = 'pagina-productos';
        $this->taxonomy = 'product_cat';

    }

    public function init() {

        // URL Rewriting
        add_action( 'init', [ $this, 'filterUrlRewriting'], 10, 0);

        add_filter( 'query_vars', [ $this, 'addParamsQueryVar' ] );

        add_filter( 'term_link', [ $this, 'changeCategoryPermalinks'], 10, 3 );

    }

    /**
     * It rewrites the URL to admit parameters for the product filters
     */
    public function filterUrlRewriting() {

        global $wp_rewrite;
        $slug_products = str_replace('/','',get_post_type_object( 'product' )->rewrite['slug']);

        // Rewrite rule for the category's page
        add_rewrite_rule('^' . $this->product_category_base . '/(.+?)/filtros-productos/(.*)',
                        'index.php?product_cat=$matches[1]&filter_products=$matches[2]',
                        'top');
        // Rewrite rule for the product's page
        add_rewrite_rule($slug_products . '/(.+?)/filtros-productos/(.*)',
                        'index.php?product=$matches[1]&filter_products=$matches[2]',
                        'top');

        $wp_rewrite->flush_rules(true);

    }

    // Add `Params` to query vars
    public function addParamsQueryVar( $vars )
    {
        $vars[] = "filter_products";
        $vars[] = "target";
        return $vars;
    }

    /**
     * It parses a string with the filter params from the URL
     *
     * @param string $filter
     *
     * @return array - [ 'families' => [], 'order' => order, 'page' => page]
     */
    public function parseFilterProducts( $filter ) {

        $arrayFilters = explode( '/', $filter );
        $result = [];

        $hasFamilies = array_search( $this->slug_families, $arrayFilters );
        $hasOrder = array_search( $this->slug_order, $arrayFilters );
        $hasPage = array_search( $this->slug_page, $arrayFilters );

        if( $hasFamilies !== false ) {

            if( $hasOrder !== false ) {

                for( $i = ( $hasFamilies + 1 ); $i < $hasOrder ; $i++ ) {
                    $result['families'][] = $arrayFilters[ $i ];
                }

                $result['order'] = $arrayFilters[ $hasOrder + 1 ];

                if( $hasPage !== false ) {

                    $result['page'] = $arrayFilters[ $hasPage + 1 ];

                }
                

            } elseif ( $hasPage !== false ) {

                for( $i = ( $hasFamilies + 1 ); $i < $hasPage ; $i++ ) {

                    $result['families'][] = $arrayFilters[ $i ];

                }

                $result['page'] = $arrayFilters[ $hasPage + 1 ]; 

            } else {

                for( $i = ( $hasFamilies + 1 ); $i < count( $arrayFilters ) ; $i++ ) {

                    $result['families'][] = $arrayFilters[ $i ];

                }

            }

        } elseif ( $hasOrder !== false ) {

            $result['order'] = $arrayFilters[ $hasOrder + 1 ];

            if( $hasPage !== false ) {

                $result['page'] = $arrayFilters[ $hasPage + 1 ];

            }

        } elseif ( $hasPage !== false ) {

            $result['page'] = $arrayFilters[ $hasPage + 1 ]; 

        }

        return $result;

    }

    /**
     * It edits the permalinks of the product's categories. It's a Wordpress HOOK
     */
    public function changeCategoryPermalinks( $url, $term, $taxonomy ) {

        // Get the categories for the post
        if( $taxonomy == $this->taxonomy ) {

            $productCategoriesRepo = new ProductCategoriesRepo();
            $level = $productCategoriesRepo->getCategoryLevel( $term->term_id );

            if( $level == 2 ) {

                $url = ( substr( $url, 0, strrpos( $url, '/', -2 ) ) ) . '/' . $this->slug_filters .'/' . $this->slug_families . '/';
                $terms = get_terms( [ 'taxonomy' => $this->taxonomy, 'parent' => $term->term_id, 'hide_empty' => true ] );
                foreach ( $terms as $t ) {
                    $url .= $t->slug . '/';
                }

            } else if ( $level == 3 ) {

                $url = ( substr( $url, 0, strrpos( ( substr( $url, 0, strrpos( $url, '/', -2 ) ) ), '/' ) ) ) . '/' . $this->slug_filters .'/' . $this->slug_families . '/' . $term->slug . '/';

            }

        }

        return $url;

    }

    /**
     * It gets the active filters
     * 
     * @return array - [ 'families' => [], 'order' => order, 'page' => page]
     */
    public function getActiveFilters() {
        
        // Getting filters from the URL
        global $wp_query;

        $filters = [];

        if( isset( $wp_query->query_vars['filter_products'] ) && $wp_query->query_vars['filter_products'] != '' ) {
            $filters = $this->parseFilterProducts( $wp_query->query_vars['filter_products'] );
        }

        $filters['original_string'] = $wp_query->query_vars['filter_products'];

        return $filters;
        
    }

}
