<?php

namespace WCAldi\Printers;

use WCAldi\Repositories\ProductCategoriesRepo;

class CategoryPrinters {

    public function init() {

        // AJAX functions
		add_action( 'wp_ajax_ajaxProductsInCategories', array( $this, 'ajaxProductsInCategories' ));
        add_action( 'wp_ajax_nopriv_ajaxProductsInCategories', array( $this, 'ajaxProductsInCategories' ));
        
    }

    /**
     * It prints the categories and subcategories together with their products
     */
    public function ajaxProductsInCategories() {

        check_ajax_referer( 'wc_aldi-ajax', 'security' );

        $category_id = $_POST['category_id'];
        $subcategories = ( $_POST['subcategories'] != '' ) ? explode( ',', $_POST['subcategories'] ) : [];
        $order = $_POST['order'];
        $page = intval( $_POST['page'] );
        $showSubcategories = $_POST['showSubcategories'];

        $limit = get_field( 'pr_pp', 'option' );
        $offset = ( $page - 1 ) * $limit;

        $productCategoriesRepo = new ProductCategoriesRepo();
        $productsCat = $productCategoriesRepo->getProductsInCategories( $category_id, $limit, $offset, $order, $subcategories );

        $totalProducts = $productCategoriesRepo->countProductsInCategories( $category_id, $subcategories );

        $params = $this->createParamsUrl( $productCategoriesRepo->getCategoriesSlugById( $subcategories ), $order, $page );

        $this->printProductsInCategories( $productsCat, $totalProducts, $page, $params );

        die(); // Wordpress requeriment for using AJAX

    }

    /**
     * It prints the categories and subcategories together with their products
     *
     * @param array $products
     * @param int $totalProducts - Product's amount without pagination
     * @param int $page - Number of page we will print - Default 1
     * @param string $urlParams - Extra params in the URl (optional)
     * @param boolean $showSubcategories - Whether to show the subcategories - Default true
     */
    public function printProductsInCategories( $products, $totalProducts, $page = 1, $urlParams = '', $showSubcategories = true ) {

        if( !empty( $products ) ) {
            
            echo "<div class='woocommerce-archive-categories-wrapper'>";

            echo get_component( 'woocommerce/woocommerce-archive-categories/woocommerce-archive-categories', [
                'products' => $products,
                'urlParams' => $urlParams,
                'showSubcategories' => $showSubcategories
            ] );

            numeric_pagination_products( $totalProducts, $page );

            echo "</div>";

        } elseif ( ! woocommerce_product_subcategories( array( 'before' => woocommerce_product_loop_start( false ), 'after' => woocommerce_product_loop_end( false ) ) ) ) {
            /**
             * woocommerce_no_products_found hook.
             *
             * @hooked wc_no_products_found - 10
             */
            do_action( 'woocommerce_no_products_found' );
        }

    }

    /**
     * It creates parameters that will be used at the URL
     *
     * @param array $families - Name of the families in an array
     * @param string $order - Default 'Alphabetical'
     * @param int $page - Default 1
     *
     * @return string - The parameters for the URL
     */
    public function createParamsUrl( $families = [], $order = 'alphabetical', $page = 1 ) {

        $hasParams = false;
        $params = '';

        if( !empty( $families ) ) {

            $params .= 'familias-productos/';

            foreach( $families as $f ) {
                $params .= $f.'/';
            }

            $hasParams = true;

        }

        if( $order != '' ) {

            $params .= 'orden-productos/'.$order.'/';
            $hasParams = true;

        }

        if( $page != 1 ) {

            $params .= 'pagina-productos/'.$page.'/';
            $hasParams = true;

        }

        if( $hasParams ) {

            $params = 'filtros-productos/'.$params;

        }

        return $params;

    }

}
