<?php

namespace WCAldi\Repositories;

use WCAldi\Repositories\ProductStoreRepo;
use WCAldi\Repositories\ProductRepo;

class ProductCategoriesRepo {

    private $storeId;
    protected $taxonomy = 'product_cat';

    public function __construct() {

        if( isset( $_COOKIE['current_store']['id'] ) ) {
            $this->storeId = $_COOKIE['current_store']['id'];
        }

    }

    /**
     * It gets the subcategories of a category, and adds the products that belong to them
     *
     * @param int $category_id - Id of the category
     * @param int $limit - Maximum number of elements to retrieve - null for no limit - Default null
     * @param int $offset - Where to start getting the info from - Default 0
     * @param string $orderby - How to sort the products - alphabetical, alphabetical_inverse, price, price_inverse - Default alphabetical
     * @param array $families - Families to retrieve products from - Empty array for all of them - Default empty
     *
     * @return array
     */
    public function getProductsInCategories( $category_id, $limit = null, $offset = 0, $orderby = 'alphabetical', $families = [] ) {
        
        $productsCat = [];

        if( $this->storeId != '' ) {

            $categories = $this->getSubcategories( $category_id );

            $products = $this->getProductsInCategoriesBBDD( $category_id, $limit, $offset, $orderby, $families );

            if( !empty( $categories ) ) {

                $i = 0;

                foreach( $categories as $category ) {

                    $productsCat[$i]['name'] = $category['name'];
                    $productsCat[$i]['id'] = $category['id'];
                    $productsCat[$i]['slug'] = $category['slug'];
                    
                    if( !empty( $category['subcategories'] ) ) {

                        $j = 0;
                        $isSubcategoryEmpty = true;
        
                        foreach ( $category['subcategories'] as $subcategory ) {

                            $productsSubcategory = [];

                            foreach( $products as $product ) {

                                if( $product['category_id'] == $subcategory['id'] ) {
                                    $productsSubcategory[] = $product;
                                }

                            }

                            if( !empty( $productsSubcategory ) ) {
                                
                                $productsCat[$i]['subcategories'][$j]['name'] = $subcategory['name'];
                                $productsCat[$i]['subcategories'][$j]['id'] = $subcategory['id'];
                                $productsCat[$i]['subcategories'][$j]['slug'] = $subcategory['slug'];
                                $productsCat[$i]['subcategories'][$j]['products'] = $productsSubcategory;

                                $isSubcategoryEmpty = false;

                                $j++;

                            }
        
                        }
        
                    }

                    if( $isSubcategoryEmpty == false ) {
                        $i++;
                    } else {
                        unset( $productsCat[$i] );
                    }
        
                }

            }

        }

        return $productsCat;

    }

    /**
     * It gets the subcategories of a concrete category
     *
     * @param int $categoryId
     *
     * @return array - An array with all the subcategories like that - [ ['name' => name, 'id' => id, 'subcategories' => [ ['name' => name, 'id' => id ], ['name' => name, 'id' => id ] ] ] ] 
     */
     public function getSubcategories( $categoryId ) {

        global $wpdb;
        
        $subcategories = [];

        $query = "
            SELECT
                t.term_id as id,
                t.slug as slug,
                t.name as name
            FROM
                {$wpdb->prefix}terms AS t
            INNER JOIN
                {$wpdb->prefix}term_taxonomy as tt ON tt.taxonomy = 'product_cat'
                    AND tt.parent = {$categoryId}
                    AND tt.term_id = t.term_id
            LEFT JOIN
                {$wpdb->prefix}termmeta as tm ON tm.term_id = t.term_id
                    AND tm.meta_key = 'order'
            ORDER BY
                LENGTH(tm.meta_value), tm.meta_value";

        if ( $result = $wpdb->get_results( $query, ARRAY_A ) ) {

            $i = 0;

            foreach( $result as $r ) {

            $subcategories[$i] = $r;
            $query = "
                SELECT
                    t.term_id as id,
                    t.slug as slug,
                    t.name as name
                FROM
                    {$wpdb->prefix}terms AS t
                INNER JOIN
                    {$wpdb->prefix}term_taxonomy as tt ON tt.taxonomy = 'product_cat'
                        AND tt.parent = {$r['id']}
                        AND tt.term_id = t.term_id
                LEFT JOIN
                    {$wpdb->prefix}termmeta as tm ON tm.term_id = t.term_id
                        AND tm.meta_key = 'order'
                ORDER BY
                    LENGTH(tm.meta_value), tm.meta_value";


            if ( $result_sub = $wpdb->get_results( $query, ARRAY_A ) ) {

                foreach( $result_sub as $s ) {
                $subcategories[$i]['subcategories'][] = $s;
                }

            }

            $i++;
            }

        }

        return $subcategories;
        
    }

    /**
     * It gets the direct subcategories of a concrete category. Meaning only one level in depth
     *
     * @param int $categoryId
     *
     * @return array - An array with all the subcategories like that - [ ['name' => name, 'id' => id, 'subcategories' => [ ['name' => name, 'id' => id ], ['name' => name, 'id' => id ] ] ] ] 
     */
    public function getDirectSubcategories( $categoryId ) {
        
        $subcategories = [];
        $args = [
            'parent' => $categoryId,
            'hide_empty' => true
        ];

        $i = 0;

        if( $terms = get_terms( $this->taxonomy, $args ) ) {

            foreach( $terms as $t ) {

                $subcategories[$i] = [
                    'id' => $t->term_id,
                    'name' => $t->name,
                    'slug' => $t->slug,
                    'parent' => $t->parent
                ];

            }
            
        }

        return $subcategories;
        
    }

    /**
     * It gets all the products belonging to a category and its subcategories
     *
     * @param int $category_id - Id of the category
     * @param int $limit - Maximum number of elements to retrieve - null for no limit - Default null
     * @param int $offset - Where to start getting the info from - Default 0
     * @param string $orderby - How to sort the products - alphabetical, alphabetical_inverse, price, price_inverse - Default alphabetical
     * @param array $families - Families to retrieve products from - Empty array for all of them - Default empty
     *
     * @return array
     */
    public function getProductsInCategoriesBBDD( $category_id, $limit = null, $offset = 0, $orderby = 'alphabetical', $families = [] ) {

        global $wpdb;
        
        $products = [];

        if( $this->storeId != '' ) {

            $productStore = new ProductStoreRepo();
            $productRepo = new ProductRepo();

            $query = $this->makeProductsInCategoriesQuery( $category_id, $families );
                
            switch ( $orderby ) {
                case 'alphabetical':
                        $query .= ', posts.post_title ASC';
                    break;
                case 'alphabetical_inverse':
                    $query .= ', posts.post_title DESC';
                    break;
                case 'price':
                        $query .= ', ps.price ASC';
                    break;
                case 'price_inverse':
                        $query .= ', ps.price DESC';
                    break;
            }

            if( $limit != null ) {
                $query .= " LIMIT ".$offset.",".$limit;
            }

            if ( $result = $wpdb->get_results( $query, ARRAY_A ) ) {

                foreach( $result as $r ) {

                    $product_id = $r['id'];

                    if( has_post_thumbnail( $product_id ) ) {
                        $thumbnail['url'] = get_the_post_thumbnail_url( $product_id, 'medium');
                        $thumbnail_id = get_post_thumbnail_id( $product_id );
                        $thumbnail['alt'] = get_post_meta($thumbnail_id, '_wp_attachment_image_alt', true);
                    } else {
                        $thumbnail = null;
                    }

                    $products[] = [
                        'id'            => $product_id,
                        'name'          => $r['name'],
                        'description'   => $r['description'],
                        'permalink'     => get_the_permalink( $product_id ),
                        'price'         => $r['price'],
                        'price_unit'    => $r['price_unit'],
                        'attributes'    => $r['attributes'],
                        'content_unit'  => strtolower( $productRepo->getAttribute($product_id, 'Pack') ),
                        'thumbnail'     => $thumbnail,
                        'category_id'   => $r['category_id'],
                        'category_name' => $r['category_name']
                    ];

                    $i++;

                }

            }

        }

        return $products;

    }

    /**
     * It counts how many products would be for the query without pagination
     *
     * @param int $category_id
     * @param string $families - String with the id of the families
     */
    public function countProductsInCategories( $category_id, $families = [] ) {

        global $wpdb;

        if( $this->storeId != '' ) {

            $query = $this->makeProductsInCategoriesQuery( $category_id, $families, true );

            if ( $result = $wpdb->get_results( $query, ARRAY_A ) ) {
                return $result[0]['total'];
            }

        } else {
            return 0;
        }

        return 0;

    }

    /**
     * Building a query for getting the products in the categories query
     *
     * @param int $category_id
     * @param array $families - Array with the ids of the families
     * @param boolean $onlyCount - Only count instead of getting the values - Default false
     *
     * @return string - The query
     */
    public function makeProductsInCategoriesQuery( $category_id, $families, $onlyCount = false ) {

        global $wpdb;

        if( $onlyCount ) {
            $data = "COUNT(posts.ID) as `total`";
        } else {
            $data = "posts.ID AS `id`,
                posts.post_title AS `name`,
                posts.post_content AS `description`,
                ps.price AS `price`,
                ps.price_unit AS `price_unit`,
                attributes.meta_value AS `attributes`,
                term_taxonomy.term_id AS `category_id`,
                {$wpdb->prefix}terms.name AS `category_name`";
        }

        if( empty( $families ) ) {
            $query_families = "AND parent IN ( SELECT term_id FROM {$wpdb->prefix}term_taxonomy WHERE parent = {$category_id} )";
        } else {
            $query_families = "AND {$wpdb->prefix}terms.term_id IN ( " . implode(",", $families) . " )";
        }

        $query = "
            SELECT
                {$data}
            FROM
                {$wpdb->prefix}term_taxonomy AS term_taxonomy
            LEFT JOIN
                {$wpdb->prefix}terms ON term_taxonomy.term_id = {$wpdb->prefix}terms.term_id
            LEFT JOIN
                {$wpdb->prefix}term_relationships AS term_relationships ON term_relationships.term_taxonomy_id = term_taxonomy.term_id
            LEFT JOIN
                {$wpdb->posts} AS posts ON term_relationships.object_id = posts.id
            LEFT JOIN
                {$wpdb->prefix}wcaldi_product_store AS ps ON ps.product_id = posts.ID
            INNER JOIN
                {$wpdb->postmeta} as postmeta ON ( posts.ID = postmeta.post_id )
            LEFT JOIN
                {$wpdb->postmeta} as attributes ON ( posts.ID = attributes.post_id AND attributes.meta_key = '_product_attributes' ) 
            LEFT JOIN
                {$wpdb->prefix}termmeta as tm ON tm.term_id = {$wpdb->prefix}terms.term_id
                    AND tm.meta_key = 'order'
            WHERE
                taxonomy = 'product_cat'
                {$query_families}
                AND ( ( postmeta.meta_key = '_visibility'
                        AND postmeta.meta_value IN ( 'catalog','visible') ) )
                AND posts.post_type = 'product'
                AND ( ( posts.post_status = 'publish' ) )
                AND ( ( SELECT is_available
                        FROM {$wpdb->prefix}wcaldi_product_store AS ps
                        WHERE ps.product_id = posts.ID
                        AND ps.store_id = {$this->storeId} ) = 1 )
                AND ps.store_id = {$this->storeId}
            ORDER BY
                LENGTH(tm.meta_value), tm.meta_value";

        return $query;

    }

    /**
     * It returns an array with the ids of the categories
     *
     * @param array $categorySlugs
     *
     * @return array
     */
    public function getCategoriesIdBySlug( $categorySlugs ) {

        $ids = [];

        foreach( $categorySlugs as $c ) {
            $ids[] = get_term_by( 'slug', $c, 'product_cat' )->term_id;
        }

        return $ids;

    }

    /**
     * It returns an array with the slugs of the categories
     *
     * @param array $categoryIds
     *
     * @return array
     */
     public function getCategoriesSlugById( $categoryIds ) {
        
        $slugs = [];

        foreach( $categoryIds as $c ) {
            $slugs[] = get_term_by( 'id', $c, 'product_cat' )->slug;
        }

        return $slugs;

    }

    /**
     * It gets the products inside a category
     *
     * @param int $category_id - Id of the category
     * @param int $number - Number of products to retrieve. null for all of them. Default null
     *
     * @return array
     */
    public function getProductsCategory( $category_id, $number = 0 ) {

        $args = array(
            'post_type'             => 'product',
            'post_status'           => 'publish',
            'ignore_sticky_posts'   => 1,
            'meta_query'            => array(
                array(
                    'key'           => '_visibility',
                    'value'         => array('catalog', 'visible'),
                    'compare'       => 'IN'
                )
            ),
            'tax_query'             => array(
                array(
                    'taxonomy'      => 'product_cat',
                    'field'         => 'term_id', //This is optional, as it defaults to 'term_id'
                    'terms'         => $category_id,
                    'operator'      => 'IN' // Possible values are 'IN', 'NOT IN', 'AND'.
                )
            )
        );

        if( $number != null ) {
            $args['posts_per_page'] = $number;
        }

        $products = new \WP_Query( $args );
        $productStore = new ProductStoreRepo();
        $productRepo = new ProductRepo();

        if( $products->have_posts() ) {

            $i = 0;
            
            while( $products->have_posts() ) { $products->the_post();

                $product_id = get_the_id();

                $dataProductStore = $productStore->statusPriceProductInStore( $product_id, $this->storeId );

                if( $dataProductStore != null && $dataProductStore['is_available'] ) {

                    $finalProducts[$i]['id'] =$product_id;
                    $finalProducts[$i]['name'] = get_the_title();
                    $finalProducts[$i]['description'] = get_the_content();
                    $finalProducts[$i]['permalink'] = get_the_permalink();
                    $finalProducts[$i]['is_available'] = $dataProductStore['is_available'];
                    $finalProducts[$i]['price'] = $dataProductStore['price'];
                    $finalProducts[$i]['price_unit'] = $dataProductStore['price_unit'];
                    $finalProducts[$i]['content_unit'] = strtolower( $productRepo->getAttribute($product_id, 'Pack') );

                    if( has_post_thumbnail( $product_id ) ) {
                        $finalProducts[$i]['thumbnail']['url'] = get_the_post_thumbnail_url( $product_id, 'medium');
                        $thumbnail_id = get_post_thumbnail_id( $product_id );
                        $finalProducts[$i]['thumbnail']['alt'] = get_post_meta($thumbnail_id, '_wp_attachment_image_alt', true);
                    } else {
                        $finalProducts[$i]['thumbnail'] = null;
                    }

                    $i++;

                }
                
            }

        }

        return $finalProducts;

    }

    /**
     * It gets the products inside a category
     *
     * @param int $category_id
     * @param int $limit - Maximum number of elements to retrieve - null for no limit - Default null
     * @param int $offset - Where to start getting the info from - Default 0
     * @param boolean $random - If true it selects by random order
     */
    public function getProductsCategoryBBDD( $category_id, $limit = null, $offset = 0, $random = false ) {

        global $wpdb;

        $products = [];

        if( $this->storeId != '' ) {

            $productStore = new ProductStoreRepo();
            $productRepo = new ProductRepo();
            
            $query = "
                SELECT
                    posts.ID AS `id`,
                    posts.post_content AS `description`,
                    posts.post_title AS `name`,
                    ps.price AS `price`,
                    ps.price_unit AS `price_unit`
                FROM {$wpdb->posts} as posts
                LEFT JOIN {$wpdb->prefix}term_relationships ON (posts.ID = {$wpdb->prefix}term_relationships.object_id)
                LEFT JOIN {$wpdb->prefix}wcaldi_product_store AS ps ON ps.product_id = posts.ID AND ps.store_id = {$this->storeId} AND ps.is_available = 1
                INNER JOIN {$wpdb->postmeta} as postmeta ON ( posts.ID = postmeta.post_id )
                WHERE 1=1
                    AND ( ( {$wpdb->prefix}term_relationships.term_taxonomy_id IN (".$category_id.") )
                        OR ( {$wpdb->prefix}term_relationships.term_taxonomy_id IN ( SELECT term_taxonomy_id FROM wp_term_taxonomy WHERE parent = ".$category_id." ) ) )
                    AND ( ( postmeta.meta_key = '_visibility'
                        AND postmeta.meta_value IN ( 'catalog','visible') ) )
                    AND posts.post_type = 'product'
                    AND ( ( posts.post_status = 'publish' ) )
                    AND ( ( SELECT is_available
                            FROM {$wpdb->prefix}wcaldi_product_store AS ps
                            WHERE ps.product_id = posts.ID
                                AND ps.store_id = ".$this->storeId." ) = 1 )
                GROUP BY posts.ID";

            if( $random ) {
                $query .= " ORDER BY RAND()";
            } else {
                $query .= " ORDER BY posts.post_date DESC";
            }

            if( $limit != null ) {
                $query .= " LIMIT ".$offset.",".$limit;
            }

            if ( $result = $wpdb->get_results( $query ) ) {

                foreach( $result as $r ) {

                    $product_id = $r->id;

                    if( has_post_thumbnail( $product_id ) ) {
                        $thumbnail['url'] = get_the_post_thumbnail_url( $product_id, 'medium');
                        $thumbnail_id = get_post_thumbnail_id( $product_id );
                        $thumbnail['alt'] = get_post_meta($thumbnail_id, '_wp_attachment_image_alt', true);
                    } else {
                        $thumbnail = null;
                    }
                    
                    $products[] = [
                        'id' => $product_id,
                        'name' => $r->name,
                        'description' => $r->description,
                        'permalink' => get_the_permalink( $product_id ),
                        'thumbnail' => $thumbnail,
                        'price' => $r->price,
                        'price_unit' => $r->price_unit,
                        'content_unit' => strtolower( $productRepo->getAttribute($product_id, 'Pack') )
                    ];
                }
                return (array) $products;
                
            } else {

                return null;
                
            }

        } else {
            return null;
        }
        
    }

	/**
    * It gets all categories recursively with children
    *
    * @param $category_id int
    *
    * @return array
    */
   public function getCategoryTree( $category_id = 0 ) {

       $terms = [];

       $termsTmp = get_terms( 'product_cat', [
           'parent' => $category_id
       ] );


       foreach ( $termsTmp as $termTmp ) {
           $term             = $this->termTransformer( $termTmp );
           $term['children'] = $this->getCategoryTree( $term['id'] );
           $terms[]          = $term;
       }

       return $terms;
   }

   /**
    * @param $wp_term \WP_Term
    *
    * @return mixed
    */
   protected function termTransformer( $wp_term ) {
       $term = [
           'id'       => $wp_term->term_id,
           'name'     => ucfirst( mb_strtolower( $wp_term->name ) ),
           'slug'     => $wp_term->slug,
           'parent'   => $wp_term->parent,
           'count'    => $wp_term->count,
           'order'    => $wp_term->term_order,
           'url'      => get_term_link( intval( $wp_term->term_id ) ),
           'children' => []
       ];

       return $term;
   }

   /**
    * Gets complete category tree from wp_options
    *
    * @return array
    */
   public function getCompleteCategoryTree() {
       return unserialize( get_option( 'product_category_tree' ) );
   }

   /**
    * Save complete category tree in wp_options
    */
   public function saveCompleteCategoryTree() {
       $value = serialize( $this->getCategoryTree() );
       update_option( 'product_category_tree', $value, true );
   }

   /**
    * It indicates which level the category belongs to. 1, 2, 3
    *
    * @param int $categoryId
    * @param int $level - Default 1
    *
    * @return int 1,2,3 
    */
    public function getCategoryLevel( $categoryId, $level = 1 ) {

        $term = get_term( $categoryId );

        if( $term->parent != 0 ) {

            $level = $this->getCategoryLevel( $term->parent, $level +1 );

        }

        return $level;

    }

}
