<?php

namespace WCAldi\Repositories;

use WCAldi\Repositories\ProductPromosRepo;
use WCAldi\Repositories\ProductCategoriesRepo;
use WCAldi\Repositories\SealsRepo;

class ProductRepo {

    public function insertProduct( $product ) {

        // Basic data
        $product_id = wp_insert_post( array(
            'post_title' => $product['name'],
            'post_content' => ($product['description']) ? $product['description'] : '',
            'post_status' => 'publish',
            'post_type' => 'product',
            'menu_order' => ($product['sort_value']) ? $product['sort_value'] : 0
        ) );

        $this->updateProductMeta( $product_id, $product );

        return $product_id;

    }

    public function updateProduct( $product_id, $product ) {

        // Basic data
        $product_data = array(
            'ID'  => $product_id,
            'post_title' => $product['name'],
            'post_content' => ( isset($product['description']) ) ? $product['description'] : '',
            'menu_order' => ($product['sort_value']) ? $product['sort_value'] : 0
        );

        wp_update_post( $product_data );

        $this->updateProductMeta( $product_id, $product );

    }

    public function updateProductMeta( $product_id, $product ) {

        update_post_meta( $product_id, '_visibility', 'visible' );
        update_post_meta( $product_id, '_stock_status', 'instock');
        update_post_meta( $product_id, 'total_sales', '0' );
        update_post_meta( $product_id, '_downloadable', 'no' );
        update_post_meta( $product_id, '_virtual', 'yes' );
        update_post_meta( $product_id, '_regular_price', '' );
        update_post_meta( $product_id, '_sale_price', '' );
        update_post_meta( $product_id, '_purchase_note', '' );
        update_post_meta( $product_id, '_featured', 'no' );
        update_post_meta( $product_id, '_weight', '' );
        update_post_meta( $product_id, '_length', '' );
        update_post_meta( $product_id, '_width', '' );
        update_post_meta( $product_id, '_height', '' );
        update_post_meta( $product_id, '_sku', $product['sku'] );
        update_post_meta( $product_id, '_product_attributes', array() );
        update_post_meta( $product_id, '_sale_price_dates_from', '' );
        update_post_meta( $product_id, '_sale_price_dates_to', '' );
        update_post_meta( $product_id, '_price', '' );
        update_post_meta( $product_id, '_sold_individually', '' );
        update_post_meta( $product_id, '_manage_stock', 'no' );
        update_post_meta( $product_id, '_backorders', 'no' );
        update_post_meta( $product_id, '_stock', '' );

    }

    /**
     * It uploads and attach the images to a Wordpress Product. When the position is 0 it's set like the featured image
     *
     * @param int $product_id | Internal product ID
     * @param array $images | Array with the product URL and the position - array( array('src' => url, 'position' => 0), array('src' => url, 'position' => 1) )
     */
    public function uploadImgs( $product_id, $images ) {

        $filename = basename($fileurl); // Get the filename including extension from the $fileurl e.g. myimage.jpg
    
        $source = WP_CONTENT_DIR.'/themes/twentyseventeen/images/'.$fileurl; // This is going to be where the image is located, depending on the fileurl you pass in this may not be needed
        $destination = WP_CONTENT_DIR. '/uploads/myuploadfolder/' . $filename; // Specify where we wish to upload the file, generally in the wp uploads directory
    
        copy($source, $destination); // Copy the file
    
        $filetype = wp_check_filetype($destination); // Get the mime type of the file
    
        $attachment = array( // Set up our images post data
            'guid'           => get_option('siteurl') . '/wp-content/uploads/myuploads/'.$filename, 
            'post_mime_type' => $filetype['type'],
            'post_title'     => $filename,
            'post_author'    => 1,
            'post_content'   => ''
        );
    
        $attach_id = wp_insert_attachment( $attachment, $destination, $product_id ); // Attach/upload image to the specified post id, think of this as adding a new post.
    
        $attach_data = wp_generate_attachment_metadata( $attach_id, $destination ); // Generate the necessary attachment data, filesize, height, width etc.
    
        wp_update_attachment_metadata( $attach_id, $attach_data ); // Add the above meta data data to our new image post
    
        add_post_meta($attach_id, '_wp_attachment_image_alt', $filealt); // Add the alt text to our new image post
    
        return $attach_id; // Return the images id to use in the below functions

    }

    /**
     * @param int $post_id - The id of the post that you are setting the attributes for
     * @param array[] $attributes - This needs to be an array containing ALL your attributes so it can insert them in one go
     */
    function wcproduct_set_attributes($product_id, $attributes) {
        $i = 0;
        // Loop through the attributes array
        foreach ( $attributes as $name => $value ) {
            $product_attributes[$i] = array (
                'name' => htmlspecialchars( stripslashes( $name ) ), // set attribute name
                'value' => $value, // set attribute value
                'position' => 1,
                'is_visible' => 1,
                'is_variation' => 1,
                'is_taxonomy' => 0
            );

            $i++;
        }

        // Now update the post with its new attributes
        update_post_meta($product_id, '_product_attributes', $product_attributes);
    }

    /**
     * It gets a product's metadata
     * 
     * @param int $productId
     * 
     * @return array - [ 'meta_description' => meta_description, 'meta_keyworkds' => meta_keywords ]
     */
    public function getProductMetadata( $productId ) {

        $metadata = [
            'meta_description'  => get_post_meta( $productId, '_yoast_wpseo_metadesc', true),
            'meta_keywords'     => get_post_meta( $productId, '_yoast_wpseo_metakeywords', true)
        ];

        return $metadata;

    }

    /**
     * It takes a product attribute
     *
     * @param int $product_id
     * @param string $name - Name of the attribute
     *
     * @return string - Value of the attribute
     */ 
    public function getAttribute($product_id, $name) {

        $attributes = get_post_meta($product_id, '_product_attributes');
        $attr = '';

        foreach( $attributes[0] as $t ) {
            if( $t['name'] == $name ) {
                $attr = $t['value'];
            }
        }

        return $attr;

    }

    /**
     * It takes all the attributes of a product
     *
     * @param int $product_id
     *
     * @return string - Array with the attributes
     */ 
    public function getAttributes( $product_id ) {
        
        $attributes = get_post_meta($product_id, '_product_attributes');
        $attr = '';

        return $attributes[0];

    }

    /**
     * Returns an attribute value of an array with all the attributes
     *
     * @param array $attributes
     * @param string $attribute_name
     *
     * @return string - Array with the attributes
     */ 
    public function getAttributeFromArray( $attributes, $attribute_name ) {
        
        $attr = '';

        foreach( $attributes as $t ) {
            if( $t['name'] === $attribute_name ) {
                $attr = $t['value'];
                
            }
        }

        return $attr;

    }

    /**
     * It gets the categories a product belongs to
     * 
     * @param int $productId
     * @param boolean $firstLevel - True to limit results to terms of first level (Optional), default false
     * @param int $parent - The parent ID (optional). Default null
     * 
     * @return array
     */
    public function getProductCategories( $productId, $firstLevel = false, $parent = null ) {

        $productCategories = [];

        $args = [];
        if( $firstLevel ) {
            $args['parent'] = 0;
        } else {
            if( !is_null( $parent ) ) {
                $args['parent'] = $parent;
            }
        }
        
        
        $terms = (array) wp_get_post_terms( $productId, 'product_cat', $args );

        if( !empty( $terms ) ) {

            foreach( $terms as $t ) {

                $productCategories[] = [
                    'id'            => $t->term_id,
                    'name'          => $t->name,
                    'slug'          => $t->slug,
                    'parent'        => $t->parent
                ];

            }

        }

        return $productCategories;

    }

    /**
     * It gets the family of the product (A.K.A. the last category in the category tree)
     * 
     * @param int $productId
     * 
     * @return string|null
     */
    public function getProductFamily( $productId ) {
        
        $family = null;

        $categories = $this->getProductCategories( $productId );
        $productCategoriesRepo = new ProductCategoriesRepo();

        if( !empty( $categories ) ) {

            foreach( $categories as $c ) {
                
                if( $productCategoriesRepo->getCategoryLevel( $c['id'] ) == 3 ) {
                    $family = $c['name'];
                    break;
                }
                
            }

        }

        return $family;

    }
    
    
    /**
     * Get the categories and subcategory of the product.
     *
     * @param int $productId
     * @return string|null
     */
    public function getProductCategoriesSubcategories($productId) {
        $categories = $this->getProductCategories($productId, false);

		if (empty($categories)) {
			return '';
		}


		$catStrings = [];

		foreach ($categories as $category) {
			if (!$category['parent']) {
				$catString = sanitize_title($category['name']) . '/';

				$iSubCat = 0;

				foreach ($categories as $subCategory) {
					if ($subCategory['parent'] === $category['id']) {
						$subCatString = sanitize_title($subCategory['name']) . '/';
						$catStrings[] = $catString . $subCatString;

						$iSubCat++;
					}
				}

				if (!$iSubCat) {
					$catStrings[] = $catString;
				}
			}
		}


		return implode(',', $catStrings);
    }


    /**
     * It gets the subcategories of a product
     *
     * @param int $productId
     *
     * @return array - An array with all the subcategories like that - [ ['name' => name, 'id' => id, 'subcategories' => [ ['name' => name, 'id' => id ], ['name' => name, 'id' => id ] ] ] ]
     */
    public function getSubcategories( $productId ) {

        $productCategoriesRepo = new ProductCategoriesRepo();

        $productSubcategories = [];

        $args = [];

        $terms = (array) wp_get_post_terms( $productId, 'product_cat', $args );

        if( !empty( $terms ) ) {

            foreach( $terms as $t ) {

            if( $productCategoriesRepo->getCategoryLevel($t->term_id) == 2 ) {

                $productSubcategories[] = [
                    'id'            => $t->term_id,
                    'name'          => $t->name,
                    'slug'          => $t->slug,
                    'parent'        => $t->parent
                ];

            }

            }

        }

        return $productSubcategories;

    }

    /**
     * It gets all the products in Woocommerce and return all its information
     * 
     * @param boolean $getPrices - Whether getting the prices in the array or not - Default false
     * @param int $limit - Total products to retrieve
     * 
     * @return array
     */
    public function getProductsInfo( $getPrices = false, $limit = null ) {

        $products = [];
        

        // The Query
        $args = [
            'post_type' => 'product',
            'post_status' => 'publish',
            'posts_per_page' => -1
        ];

        if( $limit != null ) {
            $args['posts_per_page'] = $limit;
        }
        
        $the_query = new \WP_Query( $args );

        // The Loop
        if ( $the_query->have_posts() ) {
            
            while ( $the_query->have_posts() ) { $the_query->the_post();
                $product = $this->getProductInfo( get_the_ID() );

                if( $product != null ) {
                  $products[] = $product;
                }
            }

            wp_reset_postdata();

        }

        return $products;

    }

    /**
     * It gets the product type
     * 
     * @param int $productId
     * 
     * @return string
     */
    public function getProductType( $productId ) {

        $product_type = 'Fijo';
        $promos = [];

        $productPromosRepo = new ProductPromosRepo();
        $promos = $productPromosRepo->getProductPromos( $productId, false );

        foreach( $promos as $p ) {

            if( $p['slug'] === "ofertas-semana" ) {
                $product_type = 'Fijo';
                break;
            }

            $product_type = 'Inandout';

        }

        return $product_type;

    }

    /**
     * It gets the all the information of a product from Woocommerce
     * 
     * @param mixed $productId
     * @param boolean $getPrices - Whether getting the prices in the array or not - Default false
     * 
     * @return array 
     */
    public function getProductInfo( $productId, $getPrices = false ) {

        $product = [];

        $wooproduct = new \WC_product( $productId );
        $sealsRepo = new SealsRepo();

        $product_family = get_post_meta( $productId, '_aldi_family', true );

        $product_thumbnail = '';
        $product_gallery = [];
        $promos = [];

        // Obtaining the thumbnail
        if( has_post_thumbnail( $productId ) ) {
            $product_thumbnail = get_the_post_thumbnail_url( $productId );
        }

        // Obtaining the product's gallery
        if( $attachment_ids = $wooproduct->get_gallery_attachment_ids() ) {
             
            foreach( $attachment_ids as $attachment_id ) {
                $product_gallery[] = wp_get_attachment_url( $attachment_id );
            }

        }

        $date_modified = (array) $wooproduct->get_date_modified();

        $product = [
            'id'                    => $productId,
            'name'                  => $wooproduct->get_title(),
            'description'           => $wooproduct->get_description(),
            'excerpt'               => $wooproduct->get_short_description(),
            'product_modified'      => $date_modified['date'],
            'variety'               => $wooproduct->get_attribute('variedad'),
            'origin'                => $wooproduct->get_attribute('origen'),
            'brand'                 => $wooproduct->get_attribute('marca'),
            'unit_measure'          => $wooproduct->get_attribute('unidad-de-medida'),
            'legal_text'            => $wooproduct->get_attribute('texto-legal'),
            'unit_content'          => $wooproduct->get_attribute('contenido-unidad'),
            'nutritional_category'  => $wooproduct->get_attribute('categoria-nutricional'),
            'pack'                  => $wooproduct->get_attribute('pack'),
            'outstanding_sentence'  => $wooproduct->get_attribute('frase-destacada'),
            'sku'                   => $wooproduct->get_sku(),
            'product_family'        => $product_family,
            'featured_img'          => $product_thumbnail,
            'images'                => $product_gallery,
            'permalink'             => $wooproduct->get_permalink(),
            'order'                 => $wooproduct->get_menu_order(),
            'metatags'              => $this->getProductMetadata( $productId ),
            'price'                 => $wooproduct->get_price(),
            'product_type'          => $this->getProductType( $productId ),
            'aldi_family'           => $product_family,
            'aldi_code'             => get_post_meta( $productId, '_raw_code', true),
            'seals'                 => $sealsRepo->getProductSeals( $productId ),
            'visibility'            => $wooproduct->get_catalog_visibility()
        ];

        // Getting the categories
        if( $product_family === 'AA' ) {
            $product['categories'] = '';
        } else {
            $product['categories'] = $this->getProductCategoriesSubcategories( $productId );
        }

        // Getting the product's family
        $family = $this->getProductFamily( $productId );
        if( !is_null( $family ) ) {
            $product['web_family'] = $family;
        }

        // Retrieving the promos
        $productPromosRepo = new ProductPromosRepo();
        $promo = ' ';

        if( !empty( $promos = $productPromosRepo->getProductPromos( $productId ) ) ) {

            $viewStartDate = '';
            $viewEndDate = '';

            $range = $productPromosRepo->getDateRangeBySlug($promo);

            $viewStartDate = date('Y-m-d H:i:\0\0', $range['view_start']);
            $viewEndDate = date('Y-m-d H:i:\0\0', $range['view_end']);

            $product['promo'] = [
                'start_date' => get_post_meta( $productId, 'wcaldi_promo_startdate', true),
                'end_date' => get_post_meta( $productId, 'wcaldi_promo_enddate', true),
                'view_start_date' => $viewStartDate,
                'view_end_date' => $viewEndDate,
                'normal_price' => get_post_meta( $productId, 'wcaldi_promo_normal_price', true),
                'offer_price' => get_post_meta( $productId, 'wcaldi_promo_offer_price', true),
                'normal_unit_price' => get_post_meta( $productId, 'wcaldi_promo_normal_unit_price', true),
                'offer_unit_price' => get_post_meta( $productId, 'wcaldi_promo_offer_unit_price', true)
            ];

        }

        /* TODO: Add the prices to the product if required */
        if( $getPrices ) {

        }

        return $product;

    }

    /**
     * It retrieves the product's prices for a concrete store
     * 
     * @param $productId
     * @param $storeId
     * 
     * @return array ['price' => price, 'unit_price' => unit_price]
     */
    public function getProductPriceByStore( $productId, $storeId ) {

        global $wpdb;
        
        $query = "SELECT price, price_unit
            FROM ".WC_ALDI_PRODUCT_STORE_TABLE."
            WHERE product_id = {$productId} AND store_id = {$storeId}";

        if ( $result = $wpdb->get_results( $query, ARRAY_A ) ) {
            return [
                'price'      => $result[0]['price'],
                'unit_price' => $result[0]['price_unit']
            ];
        } else {
            return false;
        }

    }

}
