<?php

namespace WCAldi\Repositories;

class ProductStoreRepo {

	/**
	 * Delete all product store relation by productId
	 *
	 * @param $productId
	 *
	 * @return false|int
	 */
	public function deleteByProductId( $productId ) {
		global $wpdb;

		$result = $wpdb->delete( WC_ALDI_PRODUCT_STORE_TABLE, [ 'product_id' => $productId ] );

		return $result;
	}


	public function insertData($data) {
		if (empty($data)) {
			return;
		}

		global $wpdb;

		// Prepare insert query.
		$table = WC_ALDI_PRODUCT_STORE_TABLE;
		$query = '';

		// Select all rows to be insterted.
		foreach ($data as $row) {
			$isAvailable = ($row['is_available'])? 1 : 0;
			$priceUnit   = "'{$row['price_unit']}'" ?: 'NULL';

			$query .= "
				SELECT {$row['product_id']} AS product_id,
				       {$row['store_id']}   AS store_id,
				       {$isAvailable}       AS is_available,
				       {$row['price']}      AS price,
				       {$priceUnit}         AS price_unit

					UNION
			";
		}

		// Remove last UNION.
		$query = substr($query, 0, -16);

		$query = "
			INSERT
				INTO {$table}
					(product_id, store_id, is_available, price, price_unit)

					/* Select data filtered. */
					SELECT i.*
						FROM ({$query}) AS i
							/* Check that the store exists. */
							INNER JOIN {$wpdb->prefix}posts AS p
									ON p.id = i.store_id
		";

		return $wpdb->query($query);
	}


	/**
	 * Gets the product-store relation by productId
	 *
	 * @param $productId
	 *
	 * @return false|array
	 */
	public function getByProductId( $productId ) {

		global $wpdb;

		$query = "SELECT stores.post_name store_name, stores.post_title store_title, stores.store_code as store_code, stores.store_id as store_id, ps.is_available, ps.price, ps.price_unit
			FROM (SELECT p.post_name post_name, p.post_title post_title, meta_value AS store_code, post_id AS store_id FROM {$wpdb->postmeta} pm inner join {$wpdb->posts} p on p.id = pm.post_id WHERE meta_key = 'ti_cod') AS stores
			LEFT JOIN " . WC_ALDI_PRODUCT_STORE_TABLE . " AS ps ON ps.store_id = stores.store_id AND ps.product_id = " . $productId . " ORDER BY stores.store_code";

		if ( $result = $wpdb->get_results( $query, OBJECT ) ) {
			return $result;
		} else {
			return false;
		}

	}

	/**
     * It returns the status and the price of a product in a store
     *
     * @param int $product_id - Id of the product
     * @param int $store_id - Id of the store
     *
     * @return array|null - ['status' => boolean, 'price' => price]
     */
	 public function statusPriceProductInStore( $product_id, $store_id ) {
        
		global $wpdb;
		
		if( $store_id == '' ) { return null; }

        if( get_post_status( $product_id ) ) {
			
			global $wpdb;
			
			$query = "SELECT ps.is_available `is_available`, ps.price `price`, ps.price_unit `price_unit`
				FROM " . WC_ALDI_PRODUCT_STORE_TABLE . " AS ps WHERE ps.store_id = " . $store_id . " AND ps.product_id = " . $product_id;
	
			if ( $result = $wpdb->get_results( $query ) ) {

				return (array) $result[0];
				
			} else {

				return null;
				
			}
        }

        return null;
    }
}
