<?php

namespace WCAldi\Repositories;

//use WCAldi\Repositories\ProductStoreRepo;
//use WCAldi\Repositories\ProductRepo;

class SearchProductsRepo {

	private $storeId;
	protected $taxonomy = 'product_cat';

	public function __construct() {
		if (isset($_COOKIE['current_store']['id'])) {
			$this->storeId = $_COOKIE['current_store']['id'];
		}
	}

	/**
	 * Get searched products.
	 *
	 * @param string $searchWords - Text to be searched.
	 * @param int $limit - Maximum number of elements to retrieve - null for no limit - Default null
	 * @param int $offset - Where to start getting the info from - Default 0
	 *
	 * @return array
	 */
	public function searchProducts($searchWords, $limit = null, $offset = 0) {
		return $this->searchProductsBBDD($searchWords, $limit, $offset);
	}


	/**
	 * It gets all the products belonging to a category and its subcategories
	 *
	 * @param string $searchWords
	 * @param int $limit - Maximum number of elements to retrieve - Default null
	 * @param int $offset - Where to start getting the info from - Default 0
	 *
	 * @return array
	 */
	public function searchProductsBBDD($searchWords, $limit = null, $offset = 0) {
		global $wpdb;

		$query = $this->makeSearchProductsQuery($searchWords);

		if ($limit) {
			$query .= " LIMIT {$offset}, {$limit}";
		}


		$result = $wpdb->get_results($query, ARRAY_A);

		if (empty($result)) {
			return [];
		}

        $productRepo = new ProductRepo();
		$products    = [];

		foreach($result as $row) {
			if (has_post_thumbnail($row['id']) ) {
				$thumbId          = get_post_thumbnail_id($row['id']);
				$thumbnail['url'] = get_the_post_thumbnail_url($row['id'], 'medium');
				$thumbnail['alt'] = get_post_meta($thumbId, '_wp_attachment_image_alt', true);
			} else {
				$thumbnail = null;
			}

			$contentUnit = $productRepo->getAttribute($row['id'], 'Pack');

			$products[] = [
				'id'           => $row['id'],
				'name'         => $row['name'],
				'description'  => $row['description'],
				'permalink'    => get_the_permalink($row['id']),
				'price'        => $row['price'],
				'price_unit'   => $row['price_unit'],
				'content_unit' => strtolower($contentUnit),
				'thumbnail'    => $thumbnail
			];

			$i++;
		}

		return $products;
	}


	/**
	 * Ccounts how many products would be returned.
	 *
	 * @param int $category_id
	 * @param string $families - String with the id of the families
	 */
	public function countSearchProducts($searchProducts) {
		global $wpdb;

		$query  = $this->makeSearchProductsQuery($searchProducts, true);
		$result = $wpdb->get_results($query, ARRAY_A);

		if (empty($result[0]['total'])) {
			return 0;
		}

		return $result[0]['total'];
	}


	/**
	 * Builds a query for searching products.
	 *
	 * @param string $searchWords
	 * @param boolean $onlyCount - Count products instead of getting the values - Default false
	 *
	 * @return string - The query
	 */
	public function makeSearchProductsQuery($searchWords, $onlyCount = false) {

		global $wpdb;

		$productPromosRepo = new \WCAldi\Repositories\ProductPromosRepo();

		$promoDates = $productPromosRepo->getSelectDateRangesBySlug( [
			'ideas-expres-lunes',
			'ideas-expres-miercoles',
			'ideas-expres-lunes',
			'entre-plantas',
			'destacados-fin-semana-general',
			'destacados-fin-semana-frescos',
			'especiales-semana',
			'gran-tematica',
			'ofertas-semana-frescos'
		] );

		foreach ($promoDates as $promoDate) {
			$startDate = date('Y-m-d', $promoDate['life_start'] );
			$timeArray[] = "'{$startDate} 00:00:00'"; // PATCH
			$timeArray[] = "'{$startDate}'";
		}

		$timesString = implode(',', $timeArray);

		if ($onlyCount) {
			$data       = "COUNT(DISTINCT(p.id)) AS `total`";
			$sqlGroupBy = '';
		} else {
			$data = "
				p.id           AS `id`,
				p.post_title   AS `name`,
				p.post_content AS `description`,
				ps.price       AS `price`,
				ps.price_unit  AS `price_unit`
			";

			$sqlGroupBy = 'GROUP BY p.id';
		}

		$query = "
		SELECT {$data}
		FROM {$wpdb->posts} AS p
			 INNER JOIN {$wpdb->postmeta} AS pms
					 ON pms.post_id = p.id
					AND pms.meta_key = '_sku'

			 INNER JOIN {$wpdb->postmeta} AS pmrc
					 ON pmrc.post_id = p.id
					AND pmrc.meta_key = '_raw_code'

			 INNER JOIN {$wpdb->postmeta} AS pm
					 ON pm.post_id = p.id
					AND pm.meta_key = '_visibility'
					AND pm.meta_value IN ('catalog', 'visible')

			 INNER JOIN {$wpdb->prefix}wcaldi_product_store AS ps
					 ON ps.product_id = p.id
					AND ps.store_id = {$this->storeId}
					AND ps.is_available IS TRUE

			 /* Check that the product is in the date range. */
			 LEFT JOIN {$wpdb->postmeta} AS pmt
					ON pmt.post_id = p.id
				   AND pmt.meta_key = 'wcaldi_promo_startdate'

			 LEFT JOIN {$wpdb->prefix}term_relationships AS tr
					ON tr.object_id = p.id

			 LEFT JOIN {$wpdb->prefix}terms AS t
					ON t.term_id = tr.term_taxonomy_id
				   AND t.name LIKE '%{$searchWords}%'

	   WHERE (p.post_title LIKE '%{$searchWords}%'
		   OR p.post_content LIKE '%{$searchWords}%'
		   OR pms.meta_value = '{$searchWords}'
		   OR pmrc.meta_value = '{$searchWords}'
		   OR t.name IS NOT NULL)
		   AND p.post_type = 'product'
		   AND p.post_status = 'publish'
		   AND (pmt.meta_value = ''
		   OR pmt.meta_value IN ({$timesString}))

	   {$sqlGroupBy}
	   ORDER BY SUBSTRING(p.post_title, 1, LENGTH('{$searchWords}')) = '{$searchWords}' DESC,
				SUBSTRING(t.name, 1, LENGTH('{$searchWords}')) = '{$searchWords}' DESC,
						  p.post_title LIKE '%{$searchWords}%' DESC,
				t.name ASC,
				p.post_title ASC
		";

		return $query;
	}
}
