<?php

namespace WCAldi\Repositories;

class StoreRepo {

	protected $stores;

	public function getAll() {

		global $wpdb;

		$query = "
			SELECT stores.*, store_codes.store_code FROM
	        {$wpdb->posts} AS stores
		        LEFT JOIN
		    		(SELECT meta_value AS store_code, post_id AS store_id FROM {$wpdb->postmeta} WHERE meta_key = 'ti_cod') 
		    		AS store_codes ON stores.ID = store_codes.store_id
			WHERE
			    post_type = 'tienda'
		";

		$results = $wpdb->get_results( $query, OBJECT );


		return $results;
	}

	public function getCodeIdRelation() {

		$stores = $this->getAll();

		$codes = [];
		foreach ( $stores as $store ) {
			$codes[ $store->store_code ] = $store->ID;
		}

		return $codes;
	}

	/**
	 * It gets all the stores with all their metadata
	 * 
	 * @param int @number_stores - Limit of stores to obtain
	 * @param string $order - 'none', 'ID', 'author', 'title'... and any possible field on a Wordpress post type
	 */
	
	public function getAllInfo( $number_stores = null, $order = null ) {

		$stores = [];
		$args = [
			'post_type' => 'tienda'
		];

		// Checking if there's a limit of stores to retrieve
		if( $number_stores != null ) {
			$args['numberposts'] = $number_stores;
		} else {
			$args['numberposts'] = -1;
		}

		if( $order != null ) {
			$args['orderby'] = $order;
			$args['order'] = 'ASC';
		}

		$query = get_posts( $args );

		foreach( $query as $s ) { setup_postdata( $s );
			$stores[] = [
				'id' => $s->ID,
				'name' => get_the_title( $s->ID ),
				'content' => get_the_content(),
				'address' => get_field( 'ti_dir', $s->ID ),
				'code' => get_field( 'ti_cod', $s->ID ),
				'latitude' => get_field( 'ti_lat', $s->ID ),
				'longitude' => get_field( 'ti_lon', $s->ID ),
				'postal_code' => get_field( 'ti_cp', $s->ID ),
				'location' => get_field( 'ti_loc', $s->ID ),
				'telephone' => get_field( 'ti_tel', $s->ID ),
				'schedule' => get_field( 'ti_hor', $s->ID ),
				'special_schedule' => get_field( 'ti_fes', $s->ID ),
				'opening' => get_field( 'ti_nue', $s->ID ),
				'province' => get_field( 'ti_pro', $s->ID ),
				'platform' => substr(get_field( 'ti_cod', $s->ID ), 0, 3),
				'isBaleares' =>	( get_field( 'ti_baleares', $s->ID ) ) ? '1' : '0'
			];
		}

		wp_reset_postdata();

		if( !empty( $stores ) ) {
			return $stores;
		} else {
			return false;
		}
		

	}

	/**
	 * It says whether the current store belongs to Baleares.
	 * It gets the value stored in the cookie.
	 * 
	 * @return boolean
	 */
	public function isBaleares() {

		if( isset( $_COOKIE['current_store']['isBaleares'] ) ) {
			$isBaleares = $_COOKIE['current_store']['isBaleares'];

			return ( $isBaleares == 1 || $isBaleares == '' ) ? true : false;
		} else {
			return false;
		}

	}

}
