<?php

namespace WCAldi\Repositories;

class TaxonomyRepo {

    protected $taxonomies;

    /**
     * It adds a product to different taxonomies
     *
     * @param int $product_id
     * @param array $taxonomies - An array of names of the promos. Use '|' for hierarchy. For example 'Tax 1|Subtax 1-1|Subsubtax 1-1-1'
     * @param string $taxonomy_type - The name of the taxonomy
     *
     * @return boolean
     */
    public function setProductTaxonomies( $product_id, $taxonomies, $taxonomy_type ) {

        $promo_ids = [];

        if( !empty($taxonomies) ) {

            foreach( $taxonomies as $t ) {

                $t = (object) $t;
                
                $parent_id = null;
                $tax_h = explode('|', $t->name);

                foreach ( $tax_h as $th ) {

                    $th = trim( $th );

                    if( $tax_data = wcaldi_get_taxonomies_by_name( $th, $taxonomy_type) ) { // Checking if it exists before

                        // Retrieving the previous terms and deciding which one is ours
                        foreach( $tax_data as $td ) {
                            
                            $my_tax = false;
                            if( $parent_id == null ) { // If the parent is null we get the first case, as we don't have more clues about who this can be
                                $my_tax = $td;
                                break;
                            } else { // If the parent isn't null, we need to find out which one is our 
                                
                                if( $td->parent == $parent_id ) { // If the parent id is equal to the one we stored, we have our taxonomy
                                    $my_tax = $td;
                                    break;
                                }

                            }
                            
                        }

                        if( $my_tax ) {
                            $tax_ids[] = $my_tax->term_id;
                            $parent_id = $my_tax->term_id;
                        } else {
                            // We create a new term
                            if( $th != '' && $th != null ) {
                                $tax_id = $this->createTaxonomyTerm($th, $taxonomy_type, $parent_id );
                                $tax_ids[] = $tax_id;
                                $parent_id = $tax_id;
                              }

                        }

                    } else {
                        // If the term wasn't found, we create it and we add it to the list retrieving the ID
                        if( $th != '' && $th != null ) {
                            $tax_id = $this->createTaxonomyTerm($th, $taxonomy_type, $parent_id );
                            $tax_ids[] = $tax_id;
                            $parent_id = $tax_id;
                          }

                    }
                    
                }
                
            }

            if( wp_set_post_terms( $product_id, $tax_ids, $taxonomy_type) ) {
                return true;
            } else {
                return false;
            }

            return true;
        } else { // If it's empty, we delete all the taxonomy data

            wp_delete_object_term_relationships( $product_id, $taxonomy_type); // Deleting taxonomy

            return true;
        }
        
        
    }

    /**
     * It creates a new term for the taxonomy
     *
     * @param string $name - The name of the element
     * @param string $taxonomy_type - The taxonomy to attach the value to
     * @param int $parent_id - The ID of the parent
     * 
     * @return false | ID of the new term inside the database
     */
    public function createTaxonomyTerm( $name, $taxonomy_type, $parent_id ) {

        if( $name != null && $name != '' ) {
            // Controlling parents
            if( $parent_id != null ) {
                $args['parent'] = $parent_id;
            } else {
                $args = [];
            }

            if( $new_tax = wp_insert_term( $name, $taxonomy_type, $args ) ) {
                $tax_id = $new_tax['term_id'];
            } else {
                return false;
            }

            return $tax_id;
        }

        return null;
    }

}