jQuery(document).ready(function () {

    if (filterExists()) {
        toggleCategories();
        applyFilters();
        changePageArchiveProducts();
        openToggleSelectedFilters();
    }

});

/**
 * It checks whether the filter exists
 */
function filterExists() {

    if (jQuery('.wcaldi-archive-filter').length) {
        return true;
    }
    return false

}

/**
 * It manages if the subcategories should be opened or not
 */
function toggleCategories() {

    jQuery('.wcaldi-archive-filter-body-category').on('click', function () {

        var arrow = jQuery(this).find('.arrow');

        if (arrow.hasClass('up')) {
            arrow.removeClass('up').addClass('down');
        } else {
            arrow.removeClass('down').addClass('up');
        }

        jQuery(this).closest('li').find('ul.wcaldi-archive-filter-body-subcategories').slideToggle();

    });

}

function applyFilters() {

    jQuery('.wcaldi-archive-filter-body-subcategories > li > input').on('change', function () {

        resetPaginationProducts();

    });

    jQuery(document).change('.woocommerce-archive-categories-sort-selector', function () {

        resetPaginationProducts();
        applyFilterAjax();
        addFilterParamsToUrl();

    });

}

/**
 * It opens all the closed subcategories that are selected at the beginning
 */
function openToggleSelectedFilters() {

    var topCat;

    jQuery('.wcaldi-archive-filter-body-subcategories input').each( function() {

        if( jQuery(this).prop('checked') ) {

            topCat = jQuery(this).closest('.wcaldi-archive-filter-body-subcategories');

            if( topCat.css('display') == 'none' ) {
                topCat.toggle();
            }
            
        }

    });

}

/**
 * It adds the subcategories to the URL
 */
function addFilterParamsToUrl() {

    var subcategory_slugs = getSlugActiveSubcategories();
    var page = getCurrentPageProducts();
    var order = getCurrentOrder();

    var base_filters = 'filtros-productos';
    var params = '';
    var hasFilters = false;
    var base_cats_slug = 'familias-productos';
    var base_page_slug = 'pagina-productos';
    var base_order_slug = 'orden-productos';
    var first_param = '';
    var total_params = '';

    if( subcategory_slugs != '' ) {
        hasFilters = true;
        total_params += base_cats_slug + "/" + subcategory_slugs.replace(/,/g, "/") + "/";
    }

    if( order != 'alphabetical' ) {
        hasFilters = true;
        total_params += base_order_slug + "/" + order + "/";
    }

    if( page != 1 ) {
        hasFilters = true;
        total_params += base_page_slug + "/" + page + "/";
    }

    if( hasFilters ) {
        params += base_filters + '/' + total_params;
    }
    
    var base_url = window.location.href; // Getting the URL

    if ( ( base_url.indexOf("/" + base_filters + "/") != -1 ) ) {
        
        var urlNoParams = base_url.substring( 0, ( base_url.indexOf("/" + base_filters + "/") ) ) + "/";

    } else {

        var urlNoParams = base_url;

    }

    history.pushState( null, "", urlNoParams + params );

}

function applyFilterAjax() {

    var category_id = jQuery('.wcaldi-archive-filter-body').data('current-cat');
    var subcategory_ids = getActiveSubcategories();
    var order = jQuery('.woocommerce-archive-categories-sort-selector').val();
    var currentPage = getCurrentPageProducts();

    var data = {
        action: 'ajaxProductsInCategories',
        security: WC_Aldi.security,
        category_id: category_id,
        subcategories: subcategory_ids,
        order: order,
        page: currentPage
    };

    // We look for the closest stores according to the position
    jQuery.ajax({
        type: 'POST',
        data: data,
        url: WC_Aldi.ajaxurl,
        success: function (data) {

            jQuery('.woocommerce-archive-categories-wrapper').fadeOut(function () {
                jQuery(this).empty();
                jQuery(this).append(data);
                jQuery(this).fadeIn();
                setTimeout(resizeImageGallery, 500);
                goToCategoryTop();
            });

        },
        error: function (request, status, error) {
            console.log(request.responseText);
        }
    });

}

/**
 * It returns the id of the active subcategories comma separated
 */
function getActiveSubcategories() {

    var activeSubcategories = '';
    var i = 0;

    jQuery('.wcaldi-archive-filter-body-subcategories > li > input').each(function () {

        if (jQuery(this).prop('checked')) {

            if (i == 0) {
                activeSubcategories += jQuery(this).data('cat');
            } else {
                activeSubcategories += "," + jQuery(this).data('cat');
            }

            i++;

        }

    });

    return activeSubcategories;

}

/**
 * It returns the slug of the active subcategories comma separated
 */
function getSlugActiveSubcategories() {
    
    var activeSubcategories = '';
    var i = 0;

    jQuery('.wcaldi-archive-filter-body-subcategories > li > input').each(function () {

        if (jQuery(this).prop('checked')) {

            if (i == 0) {
                activeSubcategories += jQuery(this).data('slug');
            } else {
                activeSubcategories += "," + jQuery(this).data('slug');
            }

            i++;

        }

    });

    return activeSubcategories;

}

// Page selection
function changePageArchiveProducts() {

    var className = '.navigation-pag-products';

    jQuery(document).on('click', className + ' > ul > li > a', function () {

        currentPos = jQuery('.navigation-pag-products > ul > li.active');
        jQuery( className + ' > ul > li').removeClass('active');

        if ( jQuery(this).closest('li').hasClass('next-page') ) {

            // ADDING ONE PAGE
            currentPos.next().addClass('active');
        
        } else if ( jQuery(this).closest('li').hasClass('prev-page') ) {

            // REDUCING ONE PAGE
            currentPos.prev().addClass('active');

        } else {

            jQuery(this).closest('li').addClass('active');

        }

        addFilterParamsToUrl();

        applyFilterAjax()

    });

}

function getCurrentPageProducts() {

    if( jQuery('.navigation-pag-products').length ) {

        if( jQuery('.navigation-pag-products > ul > li.active').length ) {
            return parseInt( jQuery('.navigation-pag-products > ul > li.active').text() );
        } else {
            return 1;
        }
            
    } else {
        return 1;
    }
    
}

function getCurrentOrder() {

    var order = 'alphabetical';

    if( jQuery('.woocommerce-archive-categories-sort-selector').length ) {

        order = jQuery('.woocommerce-archive-categories-sort-selector').val();

    }

    return order;

}

function goToCategoryTop() {

    jQuery('html,body').animate({
        scrollTop: jQuery("#products-header-banner").offset().top-170},
    'slow');

}

function resetPaginationProducts() {

    var className = '.navigation-pag-products';

    if( jQuery( className ).length ) {
        jQuery( className + ' > ul > li').removeClass('active');
    }

}